import Vue from 'vue'
import Router from 'vue-router'
import NProgress from 'nprogress'

Vue.use(Router)

const router = new Router({
  routes: [{
    path: '/login',
    name: 'Login',
    component: () => import(`@/views/login`),
    meta: {title: '登录'}
  }, {
    path: '/logout',
    name: 'Logout',
    component: () => import(`@/views/logout`),
    meta: {title: '退出登录'}
  }, {
      path: '/403',
      name: '403',
      component: () => import(`@/views/exception/403`),
      meta: {title: '403'}
    }, {
    path: '/',
    redirect: '/index'
  }, {
    path: '/index',
    name: 'index',
    component: () => import(`@/views/index`),
    meta: {title: '首页'}
  }, {
    path: '/main',
    name: 'main',
    component: () => import(`@/views/main`),
    children: [{
      path: '/search',
      name: 'search',
      component: () => import(`@/views/search`),
      meta: {title: '搜索'}
    }, {
      path: '/main/new',
      name: 'new',
      component: () => import('@/views/new/index'),
      meta: {title: '新闻资讯'}
    }, {
      path: '/main/new/:newId',
      name: 'newDetail',
      component: () => import('@/views/new/detail'),
      meta: {title: '新闻资讯'}
    }, {
      path: '/main/vr',
      name: 'vr',
      component: () => import('@/views/vr/index'),
      meta: {title: 'VR全景图'}
    }, {
      path: '/redit/:path/:searchValue',
      name: 'redit',
      component: () => import('@/views/redit')
    }, {
      path: '/main/course',
      name: 'course',
      component: () => import('@/views/course/index'),
      meta: {title: '课程介绍'}
    }, {
      path: '/main/course/:courseId',
      name: 'courseDetail',
      component: () => import('@/views/course/detail'),
      meta: {title: '课程详情'}
    }]
  }]
})

router.beforeEach((to, from, next) => {
  next()
  NProgress.start() // 每次切换页面时，调用进度条
})

router.afterEach(() => {
  NProgress.done() // 在即将进入新的页面组件前，关闭掉进度条
})

export default router
