package cn.com.sailfish.linghang.config;

import io.swagger.annotations.Api;
import lombok.extern.slf4j.Slf4j;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Profile;
import org.springframework.http.HttpHeaders;
import org.springframework.web.servlet.config.annotation.CorsRegistry;
import org.springframework.web.servlet.config.annotation.ResourceHandlerRegistry;
import org.springframework.web.servlet.config.annotation.WebMvcConfigurer;
import springfox.documentation.builders.ApiInfoBuilder;
import springfox.documentation.builders.ParameterBuilder;
import springfox.documentation.builders.PathSelectors;
import springfox.documentation.builders.RequestHandlerSelectors;
import springfox.documentation.schema.ModelRef;
import springfox.documentation.service.ApiInfo;
import springfox.documentation.service.Parameter;
import springfox.documentation.spi.DocumentationType;
import springfox.documentation.spring.web.plugins.Docket;
import springfox.documentation.swagger2.annotations.EnableSwagger2;

import java.util.ArrayList;
import java.util.List;

/**
 * Author: huangpeilin
 * Create at: 2018-12-12 11:29:33
 * Description:
 *
 * @author huangpeilin
 */
@Slf4j
@Profile({"dev", "test"})
@Configuration
@EnableSwagger2
public class SwaggerConfig implements WebMvcConfigurer {

    @Override
    public void addResourceHandlers(ResourceHandlerRegistry registry) {
        registry.addResourceHandler("swagger-ui.html").addResourceLocations("classpath:/META-INF/resources/");
        registry.addResourceHandler("/webjars/**").addResourceLocations("classpath:/META-INF/resources/webjars/");
        registry.addResourceHandler("/web_frontend/**").addResourceLocations("classpath:/web_frontend/");
    }

    @Bean
    public Docket createRestApi() {
        ParameterBuilder tokenPar = new ParameterBuilder();
        List<Parameter> pars = new ArrayList<Parameter>();
        tokenPar.name("Authorization")
                .description("令牌, Bearer 开头")
                .modelRef(new ModelRef("string"))
                .parameterType("header")
                .defaultValue("Bearer")
                .required(false)
                .build();
        pars.add(tokenPar.build());

        return new Docket(DocumentationType.SWAGGER_2)
                .apiInfo(apiInfo())
                .globalOperationParameters(pars)
                .select()
                .apis(RequestHandlerSelectors.withClassAnnotation(Api.class)) // 拦截的包路径
                .paths(PathSelectors.regex("(/web/.*)|(/admin/.*)")) // 拦截的接口路径
                .build();
    }

    private ApiInfo apiInfo() {
        return new ApiInfoBuilder()
                .title("Swagger REST API")
                .description("领航系统后台接口")
                .termsOfServiceUrl("")
                .version("1.0")
                .build();
    }

    /**
     * 功能描述：
     * 结合Security之后，还需要在SecurityConfig中设置`and().cors().and().csrf().disable()`
     *
     * @param registry
     */
    @Override
    public void addCorsMappings(CorsRegistry registry) {
        registry.addMapping("/**") // 设置允许跨域的路径
                .allowCredentials(true) // 是否允许证书，不再默认开启
                .allowedOrigins("*") // 设置允许跨域请求的域名
                .allowedHeaders("*") //允许任何请求头
                .allowedMethods("*") //允许任何方法（post、get等）
                .maxAge(60 * 60) // 跨域允许时间
                .exposedHeaders(HttpHeaders.SET_COOKIE).maxAge(60 * 60L); //maxAge(3600)表明在3600秒内，不需要再发送预检验请求，可以缓存该结果
    }

}
