package cn.com.sailfish.linghang.config;

import cn.com.sailfish.linghang.common.Http401UnauthorizedEntryPoint;
import cn.com.sailfish.linghang.repository.UserRepository;
import cn.com.sailfish.linghang.security.jwt.JWTConfigurer;
import cn.com.sailfish.linghang.security.jwt.TokenProvider;
import org.springframework.beans.factory.BeanInitializationException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.http.HttpMethod;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.config.annotation.authentication.builders.AuthenticationManagerBuilder;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.config.annotation.web.builders.WebSecurity;
import org.springframework.security.config.annotation.web.configuration.EnableWebSecurity;
import org.springframework.security.config.annotation.web.configuration.WebSecurityConfigurerAdapter;
import org.springframework.security.config.http.SessionCreationPolicy;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.security.crypto.password.PasswordEncoder;

import javax.annotation.PostConstruct;

import static cn.com.sailfish.linghang.security.SystemPrivilege.*;

/**
 * Author: huangpeilin
 * Create at: 2018-12-12 11:48:26
 * Description:
 *
 * @author huangpeilin
 */
@Configuration
@EnableWebSecurity
public class WebSecurityConfig extends WebSecurityConfigurerAdapter {

    @Autowired
    AuthenticationManagerBuilder authenticationManagerBuilder;

    @Autowired
    UserDetailsService userDetailsService;

    @Autowired
    TokenProvider tokenProvider;

    @Autowired
    UserRepository userRepository;

    @Bean
    public PasswordEncoder passwordEncoder() {
        return new BCryptPasswordEncoder();
    }

    @Override
    protected void configure(HttpSecurity http) throws Exception {
        http
                .cors().and().csrf().disable()
                .headers().frameOptions().disable()
                .and().sessionManagement().sessionCreationPolicy(SessionCreationPolicy.STATELESS)
                .and().authorizeRequests()
                .antMatchers("/vr_resource/**").permitAll()
                .antMatchers("/web/**").permitAll() // Portal前台不需要权限
                .antMatchers("/admin/**").permitAll() // 管理后台界面权限
//                .antMatchers("/admin/**").hasAnyAuthority(DEVELOPER, SUPER_ADMIN, ADMIN, INSTRUCTOR) // todo 管理后台界面权限
                .antMatchers("/apiv1/web/course/import").hasAnyAuthority(DEVELOPER)
                .antMatchers("/apiv1/web/check_item/import").hasAnyAuthority(DEVELOPER)
                .antMatchers("/apiv1/web/exception_item/import").hasAnyAuthority(DEVELOPER)
                .antMatchers("/apiv1/web/question_paper/import").hasAnyAuthority(DEVELOPER)
                .antMatchers("/apiv1/web/user/checkAddSuper").hasAnyAuthority(DEVELOPER, SUPER_ADMIN)
                .antMatchers("/apiv1/web/user/export").hasAnyAuthority(DEVELOPER, SUPER_ADMIN, ADMIN, INSTRUCTOR)
                .antMatchers("/apiv1/web/user/findOne").hasAnyAuthority(DEVELOPER, SUPER_ADMIN, ADMIN, INSTRUCTOR)
                .antMatchers("/apiv1/web/user/findAllByArea").hasAnyAuthority(DEVELOPER, SUPER_ADMIN, ADMIN, INSTRUCTOR)
                .antMatchers("/apiv1/web/user/findAllByCondition").hasAnyAuthority(DEVELOPER, SUPER_ADMIN, ADMIN, INSTRUCTOR)
                .antMatchers("/apiv1/web/user/recycle").hasAnyAuthority(DEVELOPER, SUPER_ADMIN, ADMIN, INSTRUCTOR)
                .antMatchers("/apiv1/web/user/getRecycle").hasAnyAuthority(DEVELOPER, SUPER_ADMIN, ADMIN, INSTRUCTOR)
                .antMatchers("/apiv1/web/user/recycle/findAll").hasAnyAuthority(DEVELOPER, SUPER_ADMIN, ADMIN, INSTRUCTOR)
                .antMatchers("/apiv1/web/user/enableAuthority").hasAnyAuthority(DEVELOPER, SUPER_ADMIN, ADMIN, INSTRUCTOR)
                .antMatchers("/apiv1/web/user/restore").hasAnyAuthority(DEVELOPER, SUPER_ADMIN, ADMIN, INSTRUCTOR)
                .antMatchers("/apiv1/web/user/importByLocomotive").hasAnyAuthority(DEVELOPER, SUPER_ADMIN, ADMIN, INSTRUCTOR)
                .antMatchers("/apiv1/web/user/findAllByAreaId").hasAnyAuthority(DEVELOPER, SUPER_ADMIN, ADMIN, INSTRUCTOR)
                .antMatchers("/apiv1/web/question_paper/**").hasAnyAuthority(DEVELOPER, SUPER_ADMIN, ADMIN, INSTRUCTOR)
                .antMatchers("/apiv1/web/exam/**").hasAnyAuthority(DEVELOPER, SUPER_ADMIN, ADMIN, INSTRUCTOR)
                .antMatchers("/apiv1/web/course/**").hasAnyAuthority(DEVELOPER, SUPER_ADMIN, ADMIN, INSTRUCTOR)
                .antMatchers("/apiv1/web/model/**").hasAnyAuthority(DEVELOPER, SUPER_ADMIN, ADMIN, INSTRUCTOR)
                .antMatchers("/apiv1/web/engine/**").hasAnyAuthority(DEVELOPER, SUPER_ADMIN, ADMIN, INSTRUCTOR)
                .antMatchers("/apiv1/web/area/**").hasAnyAuthority(DEVELOPER, SUPER_ADMIN, ADMIN, INSTRUCTOR)
                .antMatchers("/apiv1/web/operation_record/**").hasAnyAuthority(DEVELOPER, SUPER_ADMIN, ADMIN, INSTRUCTOR)
                .antMatchers("/apiv1/web/report_card/**").hasAnyAuthority(DEVELOPER, SUPER_ADMIN, ADMIN, INSTRUCTOR)
                .antMatchers("/apiv1/web/sum/**").hasAnyAuthority(DEVELOPER, SUPER_ADMIN, ADMIN, INSTRUCTOR)
                .antMatchers("/apiv1/univ/file/upload").hasAnyAuthority(DEVELOPER, SUPER_ADMIN, ADMIN, INSTRUCTOR)
                .antMatchers("/apiv1/web/user/login").permitAll()
                .antMatchers("/apiv1/vr/user/login").permitAll()
                .antMatchers("/apiv1/vr/user/sendMobileCode").permitAll()
                .antMatchers("/apiv1/web/user/sendMobileCode").permitAll()
                .antMatchers("/apiv1/univ/account/**").permitAll()
                .antMatchers("/apiv1/web/user/logout").permitAll()
                .antMatchers("/apiv1/vr/user/logout").permitAll()
                .antMatchers("/websocket").permitAll()
                .anyRequest().authenticated()
                .and()
                .apply(securityConfigurerAdapter());
    }

    @Override
    public void configure(WebSecurity web) throws Exception {
        web.ignoring()
                .antMatchers(HttpMethod.OPTIONS, "/**")
                .antMatchers("/app/**/*.{js,html}")
                .antMatchers("/i18n/**")
                .antMatchers("/content/**")
                .antMatchers("/swagger-ui.html")
                .antMatchers("/webjars/**")
                .antMatchers("/v2/**")
                .antMatchers("/swagger-resources/**")
                .antMatchers("/test/**")
                .antMatchers("/resources/vr_resource/**")
                .antMatchers("/actuator/**")
                .antMatchers("/monitor/**")
                .antMatchers("/h2/**");
    }

    @PostConstruct
    public void init() {
        try {
            authenticationManagerBuilder.userDetailsService(userDetailsService).passwordEncoder(passwordEncoder());
        } catch (Exception e) {
            throw new BeanInitializationException("Security configuration failed", e);
        }
    }

    private JWTConfigurer securityConfigurerAdapter() {
        return new JWTConfigurer(tokenProvider, userRepository);
    }

    @Bean
    public Http401UnauthorizedEntryPoint http401UnauthorizedEntryPoint() {
        return new Http401UnauthorizedEntryPoint();
    }

    @Bean
    @Override
    public AuthenticationManager authenticationManagerBean() throws Exception {
        return super.authenticationManagerBean();
    }

}