package cn.com.sailfish.linghang.controller.admin;

import cn.com.sailfish.linghang.common.RestRespDTO;
import cn.com.sailfish.linghang.dao.web.response.PageResponse;
import cn.com.sailfish.linghang.domain.BannerDetails;
import cn.com.sailfish.linghang.domain.OperateDto.BannerDetailsOperateDto;
import cn.com.sailfish.linghang.domain.OperateDto.ListDto;
import cn.com.sailfish.linghang.service.admin.BannerManageAdminService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;

@Api(description = "[管理后台]WEB banner页管理")
@RestController
@RequestMapping("/admin/banner")
public class BannerDetailsController {

    @Autowired
    private BannerManageAdminService bannerManageAdminService;

    @ApiOperation("banner查询分页")
    @PostMapping
    public RestRespDTO<PageResponse<BannerDetails>> bannerPage(@ApiParam("页数") @RequestParam Integer page,
                                                               @ApiParam("每一页的条数") @RequestParam Integer pageSize) {
        if (page < 1) {
            page = 1;
        }
        if (pageSize < 1) {
            pageSize = 5;
        }
        PageResponse<BannerDetails> bannerDetailsPageResponse = bannerManageAdminService.pagebystatue(page, pageSize);
        return new RestRespDTO().success(bannerDetailsPageResponse);
    }

    @ApiOperation("前移")
    @GetMapping("/{bannerDetailsId:\\d+}/forward")
    public RestRespDTO forward(@ApiParam("主键ID") @PathVariable long bannerDetailsId) {
        bannerManageAdminService.forward(bannerDetailsId);
        return new RestRespDTO().success();
    }

    @ApiOperation("后移")
    @GetMapping("/{bannerDetailsId:\\d+}/backward")
    public RestRespDTO Backward(@ApiParam("主键ID") @PathVariable long bannerDetailsId) {
        bannerManageAdminService.backward(bannerDetailsId);
        return new RestRespDTO().success();
    }

    @ApiOperation("根据id更改启用为Banner")
    @GetMapping("/{bannerDetailsId:\\d+}/banner-on")
    public RestRespDTO changeBannerOn(@ApiParam("主键Id") @PathVariable String bannerDetailsId) {
        bannerManageAdminService.changeBannerOn(bannerDetailsId);
        return new RestRespDTO().success();
    }

    @ApiOperation("新增/更新Banner")
    @PostMapping("/bannerInsert")
    public RestRespDTO bannerInsert(@Valid @RequestBody BannerDetailsOperateDto bannerDetailsOperateDto) {
        bannerManageAdminService.saveOrUpdate(bannerDetailsOperateDto);
        return new RestRespDTO().success();
    }

    @ApiOperation("根据id查询")
    @GetMapping("/{bannerId:\\d+}/findbyid")
    public RestRespDTO<BannerDetails> findbyId(@ApiParam("id主键查询") @PathVariable String bannerId) {
        return new RestRespDTO().success(bannerManageAdminService.findbyid(bannerId));
    }

    @ApiOperation("删除选中")
    @PostMapping("/delete")
    public RestRespDTO delete(@Valid @ApiParam("id主键查询") @RequestBody ListDto list) {
        bannerManageAdminService.delete(list);
        return new RestRespDTO().success();
    }

}
