package cn.com.sailfish.linghang.controller.admin;

import cn.com.sailfish.linghang.common.RestRespDTO;
import cn.com.sailfish.linghang.dao.web.response.PageResponse;
import cn.com.sailfish.linghang.domain.CourseIntroduce;
import cn.com.sailfish.linghang.domain.OperateDto.CourseIntroduceOperateDto;
import cn.com.sailfish.linghang.domain.OperateDto.ListDto;
import cn.com.sailfish.linghang.service.admin.CourseIntroduceAdminService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;

@RestController
@Api(description = "[管理后台]WEB 课程管理")
@RequestMapping("/admin/course")
public class CourseIntroduceController {

    @Autowired
    private CourseIntroduceAdminService courseIntroduceAdminService;

    @ApiOperation("分页查询")
    @PostMapping()
    public RestRespDTO courseIntroducePage(@RequestParam Integer page,
                                           @ApiParam("每一页的条数  默认是5") @RequestParam Integer pageSize,
                                           @ApiParam("每一页的条数  默认是5") @RequestParam(required = false) String crux) {
        if (page < 1) {
            page = 1;
        }
        if (pageSize < 1) {
            pageSize = 5;
        }

        PageResponse<CourseIntroduce> courseIntroducePageResponse = courseIntroduceAdminService.findAllByStatue(page, pageSize, crux);
        return new RestRespDTO().success(courseIntroducePageResponse);
    }

    @ApiOperation("前移")
    @GetMapping("/{courseId:\\d+}/forward")
    public RestRespDTO forward(@ApiParam("主键ID") @PathVariable long courseId) {
        courseIntroduceAdminService.forward(courseId);
        return new RestRespDTO().success();
    }

    @ApiOperation("后移")
    @GetMapping("/{courseId:\\d+}/backward")
    public RestRespDTO Backward(@ApiParam("主键ID") @PathVariable long courseId) {
        courseIntroduceAdminService.backward(courseId);
        return new RestRespDTO().success();
    }

    @ApiOperation("根据传入ID和序列号替换序列号")
    @GetMapping("/{courseId:\\d+}/changeSeqencing")
    public RestRespDTO changeSeqencing(@ApiParam("主键ID") @PathVariable long courseId, @ApiParam("要替换的序列号") @RequestParam Integer seqencing) {

        courseIntroduceAdminService.changeSeqencing(courseId, seqencing);

        return new RestRespDTO().success();
    }

    @ApiOperation("发布")
    @GetMapping("/{courseId:\\d+}/publish")
    public RestRespDTO checkboxrelease(@ApiParam("主键ID") @PathVariable long courseId) {
        courseIntroduceAdminService.checkboxrelease(courseId);
        return new RestRespDTO().success();
    }

    @ApiOperation("停止发布")
    @GetMapping("/{courseId:\\d+}/un-publish")
    public RestRespDTO checkboxnorelease(@ApiParam("主键ID") @PathVariable long courseId) {
        courseIntroduceAdminService.checkboxnorelease(courseId);
        return new RestRespDTO().success();
    }

    @ApiOperation("根据id查询")
    @GetMapping("/{courseId:\\d+}")
    public RestRespDTO findbyId(@ApiParam("id主键查询") @PathVariable String courseId) {
        return new RestRespDTO().success(courseIntroduceAdminService.findbyid(courseId));
    }

    @ApiOperation("根据id更改强推首页")
    @GetMapping("/{courseId:\\d+}/push-index")
    public RestRespDTO changePushIndex(@ApiParam("主键Id") @PathVariable String courseId) {
        try{
        courseIntroduceAdminService.changePushIndex(courseId);
        }catch (Exception e) {
            RestRespDTO restRespDTO = new RestRespDTO();
            restRespDTO.setErrorCode(256);
            restRespDTO.setErrorMessage("强推首页不能有大于4条记录");
            return restRespDTO;

        }        return new RestRespDTO().success();
    }

    @ApiOperation("根据id更改二级列表置顶")
    @GetMapping("/{courseId:\\d+}/second-top")
    public RestRespDTO changeSecondTop(@ApiParam("主键Id") @PathVariable String courseId) {
        courseIntroduceAdminService.changeSecondTop(courseId);
        return new RestRespDTO().success();
    }

    @ApiOperation("新增/更新课程")
    @PostMapping("/CourseIntroduceInsert")
    public RestRespDTO CourseIntroduceInsert(@Valid @RequestBody CourseIntroduceOperateDto courseIntroduceOperateDto) {
        courseIntroduceAdminService.saveOrUpdate(courseIntroduceOperateDto);
        return new RestRespDTO().success();
    }

    @ApiOperation("删除选中")
    @PostMapping("/delete")
    public RestRespDTO delete(@Valid @ApiParam("id主键查询") @RequestBody ListDto list) {
        courseIntroduceAdminService.delete(list);
        return new RestRespDTO().success();
    }

}
