package cn.com.sailfish.linghang.controller.admin;

import cn.com.sailfish.linghang.common.RestRespDTO;
import cn.com.sailfish.linghang.dao.web.response.PageResponse;
import cn.com.sailfish.linghang.domain.ImageDetails;
import cn.com.sailfish.linghang.domain.OperateDto.ListDto;
import cn.com.sailfish.linghang.service.admin.ImageDetailsAdminService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.validation.Valid;
import java.util.List;

@Api(description = "[管理后台]资源管理")
@CrossOrigin
@RestController
@RequestMapping(value = "/admin/image",method = RequestMethod.GET)
public class ImageDetailsController {

    @Autowired
    ImageDetailsAdminService imageDetailsAdminService;

    @ApiOperation(value = "上传图片")
    @PostMapping("/imageUpload")
    public RestRespDTO singleFileUpload(@ApiParam("文件") @RequestParam("file") MultipartFile file,
                                        @ApiParam("分类") @RequestParam("sourceType") String sourceType,
                                        @ApiParam("1:图片 2:视频") @RequestParam("type") String type) {
         try{
             imageDetailsAdminService.saveImage(file, sourceType, type);
         }
         catch (RuntimeException e){
             RestRespDTO restRespDTO = new RestRespDTO();
             restRespDTO.setErrorCode(257);
             restRespDTO.setErrorMessage("上传图片不能大于5MB");
             return restRespDTO;
         }
        return new RestRespDTO().success();
    }

    @ApiOperation(value = "根据Id删除图片")
    @PostMapping("/delete")
    public RestRespDTO deleleImg(@Valid @ApiParam("要删除的id的List") @RequestBody ListDto list) {
        imageDetailsAdminService.deleteImage(list);
        return new RestRespDTO().success();
    }

    @ApiOperation("分页查询")
    @PostMapping()
    public RestRespDTO courseIntroducePage(@ApiParam("当前页") @RequestParam Integer page,
                                           @ApiParam("每一页的条数  默认是5") @RequestParam Integer pageSize,
                                           @ApiParam("关键字") @RequestParam(required = false) String crux,
                                           @ApiParam("筛选类型 1或者空为全选 news为新闻 course为课程 vr3d为全景图/3D模型 banner为banner") @RequestParam(required = false) List<String> sourceType) {
        if (page < 1) {
            page = 1;
        }
        if (pageSize < 1) {
            pageSize = 5;
        }

        PageResponse<ImageDetails> imageDetailsPageResponse = imageDetailsAdminService.findAllByStatue(page, pageSize, crux,sourceType);
        return new RestRespDTO().success(imageDetailsPageResponse);
    }

}
