package cn.com.sailfish.linghang.controller.admin;

import cn.com.sailfish.linghang.common.RestRespDTO;
import cn.com.sailfish.linghang.dao.web.response.PageResponse;
import cn.com.sailfish.linghang.domain.NewsPaper;
import cn.com.sailfish.linghang.domain.OperateDto.ListDto;
import cn.com.sailfish.linghang.domain.OperateDto.NewsPaperOperateDto;
import cn.com.sailfish.linghang.service.admin.NewsPaperAdminService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;

@Api(description = "[管理后台]WEB 新闻页管理")
@RestController
@RequestMapping("/admin/news-paper")
public class NewsPaperController {

    @Autowired
    private NewsPaperAdminService newsPaperAdminService;

    @ApiOperation("分页查询")
    @PostMapping()
    public RestRespDTO findAllByStatueAndId(@ApiParam("当前页面") @RequestParam Integer page,
                                            @ApiParam("每一页的条数  默认是5") @RequestParam Integer pageSize,
                                            @ApiParam("关键字查询") @RequestParam(required = false) String crux) {
        if (page < 1) {
            page = 1;
        }
        if (pageSize < 1) {
            pageSize = 5;
        }

        PageResponse<NewsPaper> newspaperdto = newsPaperAdminService.findAllByStatueAndId(page, pageSize, crux);
        return new RestRespDTO().success(newspaperdto);
    }

    @ApiOperation("新增/更新新闻")
    @PostMapping("/NewsPaperInsert")
    public RestRespDTO NewsPaperInsert(@Valid @RequestBody NewsPaperOperateDto newsPaperOperateDto) {
        newsPaperAdminService.saveOrUpdate(newsPaperOperateDto);
        return new RestRespDTO().success();
    }

    @ApiOperation("删除选中")
    @PostMapping("/delete")
    public RestRespDTO delete(@Valid @ApiParam("id主键查询") @RequestBody ListDto list) {
        newsPaperAdminService.delete(list);
        return new RestRespDTO().success();
    }

    @ApiOperation("根据id查询")
    @GetMapping("/{newsPaperId:\\d+}")
    public RestRespDTO findById(@ApiParam("id主键查询") @PathVariable Long newsPaperId) {
        return new RestRespDTO().success(newsPaperAdminService.findById(newsPaperId));
    }

    @ApiOperation("前移")
    @GetMapping("/{newsPaperId:\\d+}/forward")
    public RestRespDTO forward(@ApiParam("主键ID") @PathVariable Long newsPaperId) {
        newsPaperAdminService.forward(newsPaperId);
        return new RestRespDTO().success();
    }

    @ApiOperation("后移")
    @GetMapping("/{newsPaperId:\\d+}/backward")
    public RestRespDTO backward(@ApiParam("主键ID") @PathVariable Long newsPaperId) {
        newsPaperAdminService.backward(newsPaperId);
        return new RestRespDTO().success();
    }

    @ApiOperation("发布")
    @GetMapping("/{newsPaperId:\\d+}/publish")
    public RestRespDTO publish(@ApiParam("主键ID") @PathVariable Long newsPaperId) {
        newsPaperAdminService.checkboxrelease(newsPaperId);
        return new RestRespDTO().success();
    }

    @ApiOperation("根据传入ID和序列号替换序列号")
    @GetMapping("/{newsPaperId:\\d+}/changeSeqencing")
    public RestRespDTO changeSeqencing(@ApiParam("主键ID") @PathVariable long newsPaperId, @ApiParam("要替换的序列号") @RequestParam Integer seqencing) {

        newsPaperAdminService.changeSeqencing(newsPaperId, seqencing);

        return new RestRespDTO().success();
    }

    @ApiOperation("取消发布")
    @GetMapping("/{newsPaperId:\\d+}/un-publish")
    public RestRespDTO unPublish(@ApiParam("主键ID") @PathVariable Long newsPaperId) {
        newsPaperAdminService.checkboxnorelease(newsPaperId);
        return new RestRespDTO().success();
    }

    @ApiOperation("根据id更改强推首页")
    @GetMapping("/{newsPaperId:\\d+}/push-index")
    public RestRespDTO changePushIndex(@ApiParam("主键Id") @PathVariable Long newsPaperId) {
        try {
            newsPaperAdminService.changePushIndex(newsPaperId);
        } catch (Exception e) {
            RestRespDTO restRespDTO = new RestRespDTO();
            restRespDTO.setErrorCode(256);
            restRespDTO.setErrorMessage("强推首页不能有大于3条记录");
            return restRespDTO;
        }
        return new RestRespDTO().success();
    }

    @ApiOperation("根据id更改二级列表置顶")
    @GetMapping("/{newsPaperId:\\d+}/second-top")
    public RestRespDTO changeSecondTop(@ApiParam("主键Id") @PathVariable Long newsPaperId) {
        newsPaperAdminService.changeSecondTop(newsPaperId);
        return new RestRespDTO().success();
    }

    @ApiOperation("根据id更改分类")
    @GetMapping("/{newsPaperId:\\d+}/classtype")
    public RestRespDTO classtype(@ApiParam("主键Id") @PathVariable Long newsPaperId) {
        newsPaperAdminService.changeClasstype(newsPaperId);
        return new RestRespDTO().success();
    }

}
