package cn.com.sailfish.linghang.controller.admin;

import cn.com.sailfish.linghang.common.RestRespDTO;
import cn.com.sailfish.linghang.dao.web.response.PageResponse;
import cn.com.sailfish.linghang.domain.OperateDto.ListDto;
import cn.com.sailfish.linghang.domain.OperateDto.ThdModelOperateDto;
import cn.com.sailfish.linghang.domain.ThdModel;
import cn.com.sailfish.linghang.service.admin.TheModelAdminService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;

@Api(description = "[管理后台]WEB 3D管理")
@RestController
@RequestMapping("/admin/model")
public class TheModelController {

    @Autowired
    private TheModelAdminService theModelAdminService;

    @PostMapping()
    public RestRespDTO courseIntroducePage(@ApiParam("当前页面") @RequestParam Integer page,
                                           @ApiParam("每一页的条数") @RequestParam Integer pageSize,
                                           @ApiParam("关键字") @RequestParam(required = false) String crux) {
        if (page < 1) {
            page = 1;
        }
        if (pageSize < 1) {
            pageSize = 5;
        }
        PageResponse<ThdModel> thdModelPageResponse = theModelAdminService.pagebystatue(page, pageSize, crux);
        return new RestRespDTO().success(thdModelPageResponse);
    }

    @ApiOperation("前移")
    @GetMapping("/{theModelId:\\d+}/forward")
    public RestRespDTO forward(@ApiParam("主键ID") @PathVariable long theModelId) {
        theModelAdminService.forward(theModelId);
        return new RestRespDTO().success();
    }

    @ApiOperation("根据传入ID和序列号替换序列号")
    @GetMapping("/{theModelId:\\d+}/changeSeqencing")
    public RestRespDTO changeSeqencing(@ApiParam("主键ID") @PathVariable long theModelId, @ApiParam("要替换的序列号") @RequestParam Integer seqencing) {

        theModelAdminService.changeSeqencing(theModelId, seqencing);

        return new RestRespDTO().success();
    }

    @ApiOperation("后移")
    @GetMapping("/{theModelId:\\d+}/backward")
    public RestRespDTO Backward(@ApiParam("主键ID") @PathVariable long theModelId) {
        theModelAdminService.backward(theModelId);
        return new RestRespDTO().success();
    }

    @ApiOperation("发布")
    @GetMapping("/{theModelId:\\d+}/publish")
    public RestRespDTO checkboxrelease(@ApiParam("主键ID") @PathVariable long theModelId) {
        theModelAdminService.checkboxrelease(theModelId);
        return new RestRespDTO().success();
    }

    @ApiOperation("取消发布")
    @GetMapping("/{theModelId:\\d+}/un-publish")
    public RestRespDTO checkboxnorelease(@ApiParam("主键ID") @PathVariable long theModelId) {
        theModelAdminService.checkboxnorelease(theModelId);
        return new RestRespDTO().success();
    }

    @ApiOperation("根据id查询")
    @GetMapping("/{theModelId:\\d+}")
    public RestRespDTO findById(@ApiParam("id主键查询") @PathVariable String theModelId) {
        return new RestRespDTO().success(theModelAdminService.findById(theModelId));
    }

    @ApiOperation("根据id更改强推首页")
    @GetMapping("/{theModelId:\\d+}/push-index")
    public RestRespDTO changePushIndex(@ApiParam("主键Id") @PathVariable String theModelId) {
        try {
            theModelAdminService.changePushIndex(theModelId);
        } catch (Exception e) {
            RestRespDTO restRespDTO = new RestRespDTO();
            restRespDTO.setErrorCode(256);
            restRespDTO.setErrorMessage("强推首页不能有大于4条记录");
            return restRespDTO;

        }
        return new RestRespDTO().success();
    }

    @ApiOperation("根据id更改二级列表置顶")
    @GetMapping("/{theModelId:\\d+}/second-top")
    public RestRespDTO changeSecondTop(@ApiParam("主键Id") @PathVariable String theModelId) {
        theModelAdminService.changeSecondTop(theModelId);
        return new RestRespDTO().success();
    }

    @ApiOperation("新增/更新VR全景图")
    @PostMapping("/theModelInsert")
    public RestRespDTO theModelInsert(@Valid @RequestBody ThdModelOperateDto thdModelOperateDto) {
        theModelAdminService.saveOrUpdate(thdModelOperateDto);
        return new RestRespDTO().success();
    }

    @ApiOperation("删除选中")
    @PostMapping("/delete")
    public RestRespDTO delete(@Valid @ApiParam("id主键查询") @RequestBody ListDto list) {
        theModelAdminService.delete(list);
        return new RestRespDTO().success();
    }

}
