package cn.com.sailfish.linghang.controller.admin;

import cn.com.sailfish.linghang.common.RestRespDTO;
import cn.com.sailfish.linghang.dao.web.response.PageResponse;
import cn.com.sailfish.linghang.domain.OperateDto.ListDto;
import cn.com.sailfish.linghang.domain.OperateDto.VrDetailsOperateDto;
import cn.com.sailfish.linghang.domain.VrDetails;
import cn.com.sailfish.linghang.service.admin.VrPanoramAdminService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;

@Api(description = "[管理后台]WEB vr视频管理")
@RestController
@RequestMapping("/admin/vrPanoram")
public class VrPanoramController {

    @Autowired
    private VrPanoramAdminService vrPanoramAdminService;

    @ApiOperation("vr图分页")
    @PostMapping()
    public RestRespDTO VrPanoramaPage(@ApiParam("当前页面") @RequestParam Integer page,
                                      @ApiParam("每一页的条数") @RequestParam Integer pageSize,
                                      @RequestParam(required = false) String crux) {
        if (page < 1) {
            page = 1;
        }
        if (pageSize < 1) {
            pageSize = 5;
        }
        PageResponse<VrDetails> vrDetailsPageResponse = vrPanoramAdminService.pagebystatue(page, pageSize, crux);
        return new RestRespDTO().success(vrDetailsPageResponse);
    }

    @ApiOperation("前移")
    @GetMapping("/{vrPanoramId:\\d+}/forward")
    public RestRespDTO forward(@ApiParam("主键ID") @PathVariable long vrPanoramId) {
        vrPanoramAdminService.forward(vrPanoramId);
        return new RestRespDTO().success();
    }

    @ApiOperation("根据传入ID和序列号替换序列号")
    @GetMapping("/{vrPanoramId:\\d+}/changeSeqencing")
    public RestRespDTO changeSeqencing(@ApiParam("主键ID") @PathVariable long vrPanoramId, @ApiParam("要替换的序列号") @RequestParam Integer seqencing) {

        vrPanoramAdminService.changeSeqencing(vrPanoramId, seqencing);

        return new RestRespDTO().success();
    }

    @ApiOperation("后移")
    @GetMapping("/{vrPanoramId:\\d+}/backward")
    public RestRespDTO Backward(@ApiParam("主键ID") @PathVariable long vrPanoramId) {
        vrPanoramAdminService.backward(vrPanoramId);
        return new RestRespDTO().success();
    }

    @ApiOperation("发布")
    @GetMapping("/{vrPanoramId:\\d+}/publish")
    public RestRespDTO checkboxrelease(@ApiParam("主键ID") @PathVariable long vrPanoramId) {
        vrPanoramAdminService.checkboxrelease(vrPanoramId);
        return new RestRespDTO().success();
    }

    @ApiOperation("取消发布")
    @GetMapping("/{vrPanoramId:\\d+}/un-publish")
    public RestRespDTO checkboxnorelease(@ApiParam("主键ID") @PathVariable long vrPanoramId) {
        vrPanoramAdminService.checkboxnorelease(vrPanoramId);
        return new RestRespDTO().success();
    }

    @ApiOperation("根据id查询")
    @GetMapping("/{vrPanoramId:\\d+}")
    public RestRespDTO findById(@ApiParam("id主键查询") @PathVariable String vrPanoramId) {
        return new RestRespDTO().success(vrPanoramAdminService.findById(vrPanoramId));
    }

    @ApiOperation("根据id更改强推首页")
    @GetMapping("/{vrPanoramId:\\d+}/push-index")
    public RestRespDTO changePushIndex(@ApiParam("主键Id") @PathVariable String vrPanoramId) {
        try {
            vrPanoramAdminService.changePushIndex(vrPanoramId);
        } catch (Exception e) {
            RestRespDTO restRespDTO = new RestRespDTO();
            restRespDTO.setErrorCode(256);
            restRespDTO.setErrorMessage("强推首页不能有大于4条记录");
            return restRespDTO;
        }
        return new RestRespDTO().success();
    }

    @ApiOperation("根据id更改二级列表置顶")
    @GetMapping("/{vrPanoramId:\\d+}/second-top")
    public RestRespDTO changeSecondTop(@ApiParam("主键Id") @PathVariable String vrPanoramId) {
        vrPanoramAdminService.changeSecondTop(vrPanoramId);
        return new RestRespDTO().success();
    }

    @ApiOperation("新增/更新VR全景图")
    @PostMapping("/VrDetailsInsert")
    public RestRespDTO bannerInsert(@Valid @RequestBody VrDetailsOperateDto vrDetailsOperateDto) {
        vrPanoramAdminService.saveOrUpdate(vrDetailsOperateDto);
        return new RestRespDTO().success();
    }

    @ApiOperation("删除选中")
    @PostMapping("/delete")
    public RestRespDTO delete(@Valid @ApiParam("id主键查询") @RequestBody ListDto list) {
        vrPanoramAdminService.delete(list);
        return new RestRespDTO().success();
    }

}
