package cn.com.sailfish.linghang.controller.web;

import cn.com.sailfish.linghang.common.RestRespDTO;
import cn.com.sailfish.linghang.dao.web.response.PageResponse;
import cn.com.sailfish.linghang.domain.*;
import cn.com.sailfish.linghang.service.portal.*;
import cn.com.sailfish.linghang.util.RestRespUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Map;

/**
 * Create at: 2020-1-15 14:06
 * Description:
 *
 * @author lichunjiang
 */
@Api(description = "[Portal]WEB 首页管理")
@RestController
@RequestMapping("/web")
public class WebController {

    @Autowired
    private BannerPortalService bannerPortalService;

    @Autowired
    private BannerManagePortalService bannerManagePortalService;

    @Autowired
    private NewsPaperPortalService newsPaperPortalService;

    @Autowired
    private CourseIntroducePortalService courseIntroducePortalService;

    @Autowired
    private TheModelPortalService theModelPortalService;

    @Autowired
    private VrPanoramPortalService vrPanoramPortalService;

    @Autowired
    private SearchPortalService searchPortalService;

  /*  @Deprecated
    @ApiOperation("新闻展示接口")
    @PostMapping("/news/all")
    public RestRespDTO newsAllMessage() {
        List<NewsPaper> allNewspaperlist = bannerConResource.allNewspaper();
        return RestRespUtil.getSuccessRsp(allNewspaperlist);
    }

    @Deprecated
    @ApiOperation("VR展示接口")
    @PostMapping("/ThdModel/all")
    public RestRespDTO VrShowMessage() {
        List<VrDetails> allvrDetaillist = bannerConResource.allvrDetails();
        return RestRespUtil.getSuccessRsp(allvrDetaillist);
    }

    @Deprecated
    @ApiOperation("课程介绍展示接口")
    @PostMapping("/course/all")
    public RestRespDTO courseIntroduceMessage() {
        List<CourseIntroduce> list = bannerConResource.allCourseCourseIntroduce();
        return RestRespUtil.getSuccessRsp(list);
    }
*/

    @ApiOperation("页头查询接口")
    @PostMapping("/inquire/all")
    public RestRespDTO Queryall(
            @ApiParam("crux查询关键字")
            @RequestParam(required = false) String crux,
            @ApiParam("type 1 新闻，2：课程介绍 3：vr视图 4：3D模型")
            @RequestParam String type,
            @ApiParam("当前页")
            @RequestParam Integer page,
            @ApiParam("每一页的条数  默认是5")
            @RequestParam Integer pageSize,
            @RequestParam(required = false) String newstype) {
        Map<String, PageResponse<?>> list = bannerPortalService.findBycrux(crux, type , page ,pageSize,newstype);
        return RestRespUtil.getSuccessRsp(list);
    }

    @ApiOperation("首页聚合接口")
    @PostMapping("/home")
    public RestRespDTO home() {
        Map<String, List<?>> hashmap =  bannerPortalService.bannershow();
        return RestRespUtil.getSuccessRsp(hashmap);
    }

    @ApiOperation("banner")
    @PostMapping("/banners")
    public RestRespDTO banners(
            @ApiParam("当前页面") @RequestParam Integer page,
            @ApiParam("每一页的条数 默认是7") @RequestParam Integer pageSize,
            @ApiParam("关键字查询") @RequestParam String crux) {
        if (page < 1) {
            page = 1;
        }
        if (pageSize < 1) {
            pageSize = 7;
        }
        return RestRespUtil.getSuccessRsp(bannerManagePortalService.pagebystatue(page, pageSize));
    }

    @ApiOperation("新闻分页查询")
    @PostMapping("/news")
    public RestRespDTO news(
            @ApiParam("当前页面") @RequestParam Integer page,
            @ApiParam("每一页的条数 默认是12") @RequestParam Integer pageSize,
            @ApiParam("关键字查询") @RequestParam(required = false) String crux,
            @RequestParam(required = false) String newstype) {
        if (page < 1) {
            page = 1;
        }
        if (pageSize < 1) {
            pageSize = 12;
        }

        PageResponse<NewsPaper> newspaperdto = newsPaperPortalService.findAllByStatueAndId(page, pageSize, crux,newstype);
        return RestRespUtil.getSuccessRsp(newspaperdto);
    }

    @ApiOperation("课程介绍分页查询")
    @PostMapping("/course")
    public RestRespDTO courses(
            @ApiParam("当前页面") @RequestParam Integer page,
            @ApiParam("每一页的条数 默认是12") @RequestParam Integer pageSize,
            @ApiParam("关键字查询") @RequestParam(required = false) String crux) {
        if (page < 1) {
            page = 1;
        }
        if (pageSize < 1) {
            pageSize = 12;
        }

        PageResponse<CourseIntroduce> courseIntroducePageResponse = courseIntroducePortalService.findAllByStatue(page, pageSize, crux);
        return RestRespUtil.getSuccessRsp(courseIntroducePageResponse);
    }

    @ApiOperation("3d分页")
    @PostMapping("model")
    public RestRespDTO thdModel(
            @ApiParam("当前页面") @RequestParam Integer page,
            @ApiParam("每一页的条数 默认是12") @RequestParam Integer pageSize,
            @ApiParam("关键字查询") @RequestParam(required = false) String crux) {
        if (page < 1) {
            page = 1;
        }
        if (pageSize < 1) {
            pageSize = 12;
        }

        PageResponse<ThdModel> thdModelPageResponse = theModelPortalService.pagebystatue(page, pageSize, crux);
        return RestRespUtil.getSuccessRsp(thdModelPageResponse);
    }

    @ApiOperation("vr图分页")
    @PostMapping("/vrPanoram")
    public RestRespDTO vrPanorama(
            @ApiParam("当前页面") @RequestParam Integer page,
            @ApiParam("每一页的条数 默认是12") @RequestParam Integer pageSize,
            @ApiParam("关键字查询") @RequestParam(required = false) String crux) {
        if (page < 1) {
            page = 1;
        }
        if (pageSize < 1) {
            pageSize = 12;
        }

        PageResponse<VrDetails> vrDetailsPageResponse = vrPanoramPortalService.pagebystatue(page, pageSize, crux);
        return RestRespUtil.getSuccessRsp(vrDetailsPageResponse);
    }

    @ApiOperation("根据id查询")
    @GetMapping("/course/{courseId:\\d+}")
    public RestRespDTO coursefindbyId(@ApiParam("id主键查询") @PathVariable String courseId) {
        CourseIntroduce courseIntroduce = courseIntroducePortalService.findbyid(courseId);
        return RestRespUtil.getSuccessRsp(courseIntroduce);
    }

    @ApiOperation("根据id查询")
    @GetMapping("/news/{newsPaperId:\\d+}")
    public RestRespDTO newsfindById(@ApiParam("id主键查询") @PathVariable String newsPaperId) {
        return RestRespUtil.getSuccessRsp(newsPaperPortalService.findById(newsPaperId));
    }

    @ApiOperation("根据id查询")
    @GetMapping("/vrPanoram/{vrPanoramId:\\d+}")
    public RestRespDTO vrPanoramfindById(@ApiParam("id主键查询") @PathVariable String vrPanoramId){
        return RestRespUtil.getSuccessRsp(vrPanoramPortalService.findbyid(vrPanoramId));
    }

    @ApiOperation("视图查询接口")
    @PostMapping("/view/all")
    public RestRespDTO findViewByCrux(
            @ApiParam("crux查询关键字")
            @RequestParam(required = false) String crux,
            @ApiParam("type 1 新闻资讯，2：行业信息 3：VR全景图 4：课程介绍")
            @RequestParam(required = false) String type,
            @ApiParam("当前页")
            @RequestParam Integer page,
            @ApiParam("每一页的条数  默认是5")
            @RequestParam Integer pageSize) {
        PageResponse<SelectAllFromCruxView> viewByCrux = searchPortalService.findViewByCrux(crux, type, page, pageSize);
        return RestRespUtil.getSuccessRsp(viewByCrux);
    }

}
