package cn.com.sailfish.linghang.domain;

import javax.persistence.*;
import javax.validation.constraints.Size;
import java.io.Serializable;
import java.time.Instant;
import java.util.Objects;
import java.util.List;

import cn.com.sailfish.linghang.common.auditor.Auditable;

/**
 * @author: AutoCoder (maintained by newma<newma@live.cn>)
 * @create: 2019-07-11 11:19:25
 */

/**
 * 课程、工卡信息表
 */
@Entity
@Table(name = "course")
public class Course extends Auditable implements Serializable {

    private static final long serialVersionUID = 1L;

    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    @Column(name = "id")
    private Long id;

    @Column(name = "import_id")
    private Long importId; // 课程导入表的ID值

    @Column(name = "name")
    @Size(min = 1, max = 50)
    private String name; // 课程英文名字, 长度大于1小于50, 不为空

    @Column(name = "chinese_name")
    @Size(min = 1, max = 50)
    private String chineseName; // 课程中文名称，长度大于1小于50，允许为空

    @Column(name = "division")
    private Integer division; // 部门 0:乘务; 1:机务

    @Column(name = "total_abnormals")
    private Integer totalAbnormals; // 异常总数量

    @ManyToOne
    @JoinColumn(name = "model_id")
    private Model model;

    @ManyToOne
    @JoinColumn(name = "engine_id")
    private Engine engine;

    @ManyToMany
    @JoinTable(name = "course_check_item",
                joinColumns = @JoinColumn(name = "course_id"),
                inverseJoinColumns = @JoinColumn(name = "check_item_id"))
    private List<CheckItem> checkItem;

    @ManyToMany
    @JoinTable(name = "course_exception_item",
                joinColumns = @JoinColumn(name = "course_id"),
                inverseJoinColumns = @JoinColumn(name = "exception_item_id"))
    private List<ExceptionItem> exceptionItem;



    // getter and setter

    public Long getId() {
        return this.id;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public Long getImportId() {
        return this.importId;
    }

    public void setImportId(Long importId) {
        this.importId = importId;
    }

    public String getName() {
        return this.name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getChineseName() {
        return this.chineseName;
    }

    public void setChineseName(String chineseName) {
        this.chineseName = chineseName;
    }

    public Integer getDivision() {
        return this.division;
    }

    public void setDivision(Integer division) {
        this.division = division;
    }

    public Integer getTotalAbnormals() {
        return this.totalAbnormals;
    }

    public void setTotalAbnormals(Integer totalAbnormals) {
        this.totalAbnormals = totalAbnormals;
    }

    public Model getModel() {
        return this.model;
    }

    public void setModel(Model model) {
        this.model = model;
    }

    public Engine getEngine() {
        return this.engine;
    }

    public void setEngine(Engine engine) {
        this.engine = engine;
    }

    public List<CheckItem> getCheckItem() {
        return this.checkItem;
    }

    public void setCheckItem(List<CheckItem> checkItem) {
        this.checkItem = checkItem;
    }

    public List<ExceptionItem> getExceptionItem() {
        return this.exceptionItem;
    }

    public void setExceptionItem(List<ExceptionItem> exceptionItem) {
        this.exceptionItem = exceptionItem;
    }



    @Override
    public int hashCode() {
        return Objects.hashCode(getId());
    }

    @Override
    public String toString() {
        return "{ Course: { " +
            "'id': '" + getId() + "'" +
            ", 'importId': '" + getImportId() + "'" +
            ", 'name': '" + getName() + "'" +
            ", 'chineseName': '" + getChineseName() + "'" +
            ", 'division': '" + getDivision() + "'" +
            ", 'totalAbnormals': '" + getTotalAbnormals() + "'" +
            " }}";
    }
}
