package cn.com.sailfish.linghang.domain;

import javax.persistence.*;
import javax.validation.constraints.Size;
import java.io.Serializable;
import java.time.Instant;
import java.util.Objects;
import java.util.List;

import cn.com.sailfish.linghang.common.auditor.Auditable;

/**
 * @author: AutoCoder (maintained by newma<newma@live.cn>)
 * @create: 2019-07-11 11:19:25
 */

/**
 * 操作记录表
 */
@Entity
@Table(name = "operation_record")
public class OperationRecord extends Auditable implements Serializable {

    private static final long serialVersionUID = 1L;

    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    @Column(name = "id")
    private Long id;

    @Column(name = "login_time")
    private Instant loginTime; // 登录时间

    @Column(name = "logout_time")
    private Instant logoutTime; // 登出时间

    @Column(name = "start_time")
    private Instant startTime; // 开始时间

    @Column(name = "end_time")
    private Instant endTime; // 结束时间

    @Column(name = "status", nullable = false)
    private Integer status; // 是否是最新状态 0 是 1 不是

    @Column(name = "ip")
    private String ip; // 登录的终端IP

    @Column(name = "type")
    private Integer type; // 操作的类型 1 考试  0 训练

    @Column(name = "card")
    private String card; // 用户工号

    @Column(name = "username")
    private String username; // 用户名字

    @Column(name = "exam_id")
    private Long examId; // 所属考试的ID  操作类型为考试时此处不为空

    @Column(name = "type_name")
    private String typeName; // 试卷名字或者课程名字

    @Column(name = "score")
    private Float score; // 考试成绩

    @Column(name = "assessment")
    private Integer assessment; // 评定 0 及格 1 不及格 2 通过 3 不通过

    @Column(name = "invigilator")
    private String invigilator; // 监考人集合

    @Column(name = "course_id")
    private Long courseId; // 所属课程的ID  操作类型为训练时此处不为空

    @ManyToOne
    @JoinColumn(name = "area_id")
    private Area area;

    // getter and setter

    public Long getId() {
        return this.id;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public Instant getLoginTime() {
        return this.loginTime;
    }

    public void setLoginTime(Instant loginTime) {
        this.loginTime = loginTime;
    }

    public Instant getLogoutTime() {
        return this.logoutTime;
    }

    public void setLogoutTime(Instant logoutTime) {
        this.logoutTime = logoutTime;
    }

    public Instant getStartTime() {
        return this.startTime;
    }

    public void setStartTime(Instant startTime) {
        this.startTime = startTime;
    }

    public Instant getEndTime() {
        return this.endTime;
    }

    public void setEndTime(Instant endTime) {
        this.endTime = endTime;
    }

    public Integer getStatus() {
        return this.status;
    }

    public void setStatus(Integer status) {
        this.status = status;
    }

    public String getIp() {
        return this.ip;
    }

    public void setIp(String ip) {
        this.ip = ip;
    }

    public Integer getType() {
        return this.type;
    }

    public void setType(Integer type) {
        this.type = type;
    }

    public String getCard() {
        return this.card;
    }

    public void setCard(String card) {
        this.card = card;
    }

    public String getUsername() {
        return this.username;
    }

    public void setUsername(String username) {
        this.username = username;
    }

    public Long getExamId() {
        return this.examId;
    }

    public void setExamId(Long examId) {
        this.examId = examId;
    }

    public String getTypeName() {
        return this.typeName;
    }

    public void setTypeName(String typeName) {
        this.typeName = typeName;
    }

    public Float getScore() {
        return this.score;
    }

    public void setScore(Float score) {
        this.score = score;
    }

    public Integer getAssessment() {
        return this.assessment;
    }

    public void setAssessment(Integer assessment) {
        this.assessment = assessment;
    }

    public String getInvigilator() {
        return this.invigilator;
    }

    public void setInvigilator(String invigilator) {
        this.invigilator = invigilator;
    }

    public Long getCourseId() {
        return this.courseId;
    }

    public void setCourseId(Long courseId) {
        this.courseId = courseId;
    }

    public Area getArea() {
        return this.area;
    }

    public void setArea(Area area) {
        this.area = area;
    }

    @Override
    public int hashCode() {
        return Objects.hashCode(getId());
    }

    @Override
    public String toString() {
        return "{ OperationRecord: { " +
            "'id': '" + getId() + "'" +
            ", 'loginTime': '" + getLoginTime() + "'" +
            ", 'logoutTime': '" + getLogoutTime() + "'" +
            ", 'startTime': '" + getStartTime() + "'" +
            ", 'endTime': '" + getEndTime() + "'" +
            ", 'status': '" + getStatus() + "'" +
            ", 'ip': '" + getIp() + "'" +
            ", 'type': '" + getType() + "'" +
            ", 'card': '" + getCard() + "'" +
            ", 'username': '" + getUsername() + "'" +
            ", 'examId': '" + getExamId() + "'" +
            ", 'typeName': '" + getTypeName() + "'" +
            ", 'score': '" + getScore() + "'" +
            ", 'assessment': '" + getAssessment() + "'" +
            ", 'invigilator': '" + getInvigilator() + "'" +
            ", 'courseId': '" + getCourseId() + "'" +
            " }}";
    }
}
