package cn.com.sailfish.linghang.domain;

import javax.persistence.*;
import javax.validation.constraints.Size;
import java.io.Serializable;
import java.time.Instant;
import java.util.Objects;
import java.util.List;

import cn.com.sailfish.linghang.common.auditor.Auditable;

/**
 * @author: AutoCoder (maintained by newma<newma@live.cn>)
 * @create: 2019-07-11 11:19:25
 */

/**
 * 试卷信息表
 */
@Entity
@Table(name = "question_paper")
public class QuestionPaper extends Auditable implements Serializable {

    private static final long serialVersionUID = 1L;

    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    @Column(name = "id")
    private Long id;

    @Column(name = "import_id")
    private Long importId; // 试卷导入表的ID值

    @Column(name = "title", length = 50)
    private String title; // 试卷标题

    @Column(name = "description")
    private String description; // 试卷描述

    @Column(name = "test_length")
    private Integer testLength; // 考试时长，单位是分钟

    @Column(name = "type", nullable = false)
    private Integer type; // 试卷类型  0:考试卷;  1:练习卷 默认是0

    @Column(name = "b_is_parts_of_buoy")
    private Boolean bIsPartsOfBuoy; // 是否具有部位浮标功能

    @Column(name = "b_is_location_hints")
    private Boolean bIsLocationHints; // 是否具有位置提示功能

    @Column(name = "b_is_environmental_sound")
    private Boolean bIsEnvironmentalSound; // 是否具有环境声效功能

    @Column(name = "b_is_operating_hints")
    private Boolean bIsOperatingHints; // 是否具有操作提示功能

    @Column(name = "b_is_voice_hints")
    private Boolean bIsVoiceHints; // 是否具有语音提示功能

    @Column(name = "b_is_name_tag")
    private Boolean bIsNameTag; // 是否具有名称标签功能

    @Column(name = "b_is_graphic_popup_window")
    private Boolean bIsGraphicPopupWindow; // 是否具有图文弹窗功能

    @Column(name = "b_is_warning_notice")
    private Boolean bIsWarningNotice; // 是否具有警示注意功能

    @Column(name = "b_is_delete")
    private Boolean bIsDelete; // 试卷是否已经在回收站中, 默认是false

    @Column(name = "random_abnormals")
    private Integer randomAbnormals; // 人为设定的部件异常出现数量

    @Column(name = "exception_generate_type", nullable = false)
    private Integer exceptionGenerateType; // 异常生成类型 0:随机分布; 1:均匀分布 默认为0

    @Column(name = "qualified_quantity")
    private Integer qualifiedQuantity; // 特殊项合格所需数量

    @Column(name = "score")
    private Float score; // 试卷总分

    @Column(name = "passing_score")
    private Float passingScore; // 及格分数

    @ManyToOne
    @JoinColumn(name = "author_id")
    private User author;

    @ManyToOne
    @JoinColumn(name = "course_id")
    private Course course;

    @ManyToOne
    @JoinColumn(name = "area_id")
    private Area area;

    @ManyToOne
    @JoinColumn(name = "model_id")
    private Model model;

    @ManyToOne
    @JoinColumn(name = "engine_id")
    private Engine engine;




    // getter and setter

    public Long getId() {
        return this.id;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public Long getImportId() {
        return this.importId;
    }

    public void setImportId(Long importId) {
        this.importId = importId;
    }

    public String getTitle() {
        return this.title;
    }

    public void setTitle(String title) {
        this.title = title;
    }

    public String getDescription() {
        return this.description;
    }

    public void setDescription(String description) {
        this.description = description;
    }

    public Integer getTestLength() {
        return this.testLength;
    }

    public void setTestLength(Integer testLength) {
        this.testLength = testLength;
    }

    public Integer getType() {
        return this.type;
    }

    public void setType(Integer type) {
        this.type = type;
    }

    public Boolean getBIsPartsOfBuoy() {
        return this.bIsPartsOfBuoy;
    }

    public void setBIsPartsOfBuoy(Boolean bIsPartsOfBuoy) {
        this.bIsPartsOfBuoy = bIsPartsOfBuoy;
    }

    public Boolean getBIsLocationHints() {
        return this.bIsLocationHints;
    }

    public void setBIsLocationHints(Boolean bIsLocationHints) {
        this.bIsLocationHints = bIsLocationHints;
    }

    public Boolean getBIsEnvironmentalSound() {
        return this.bIsEnvironmentalSound;
    }

    public void setBIsEnvironmentalSound(Boolean bIsEnvironmentalSound) {
        this.bIsEnvironmentalSound = bIsEnvironmentalSound;
    }

    public Boolean getBIsOperatingHints() {
        return this.bIsOperatingHints;
    }

    public void setBIsOperatingHints(Boolean bIsOperatingHints) {
        this.bIsOperatingHints = bIsOperatingHints;
    }

    public Boolean getBIsVoiceHints() {
        return this.bIsVoiceHints;
    }

    public void setBIsVoiceHints(Boolean bIsVoiceHints) {
        this.bIsVoiceHints = bIsVoiceHints;
    }

    public Boolean getBIsNameTag() {
        return this.bIsNameTag;
    }

    public void setBIsNameTag(Boolean bIsNameTag) {
        this.bIsNameTag = bIsNameTag;
    }

    public Boolean getBIsGraphicPopupWindow() {
        return this.bIsGraphicPopupWindow;
    }

    public void setBIsGraphicPopupWindow(Boolean bIsGraphicPopupWindow) {
        this.bIsGraphicPopupWindow = bIsGraphicPopupWindow;
    }

    public Boolean getBIsWarningNotice() {
        return this.bIsWarningNotice;
    }

    public void setBIsWarningNotice(Boolean bIsWarningNotice) {
        this.bIsWarningNotice = bIsWarningNotice;
    }

    public Boolean getBIsDelete() {
        return this.bIsDelete;
    }

    public void setBIsDelete(Boolean bIsDelete) {
        this.bIsDelete = bIsDelete;
    }

    public Integer getRandomAbnormals() {
        return this.randomAbnormals;
    }

    public void setRandomAbnormals(Integer randomAbnormals) {
        this.randomAbnormals = randomAbnormals;
    }

    public Integer getExceptionGenerateType() {
        return this.exceptionGenerateType;
    }

    public void setExceptionGenerateType(Integer exceptionGenerateType) {
        this.exceptionGenerateType = exceptionGenerateType;
    }

    public Integer getQualifiedQuantity() {
        return this.qualifiedQuantity;
    }

    public void setQualifiedQuantity(Integer qualifiedQuantity) {
        this.qualifiedQuantity = qualifiedQuantity;
    }

    public Float getScore() {
        return this.score;
    }

    public void setScore(Float score) {
        this.score = score;
    }

    public Float getPassingScore() {
        return this.passingScore;
    }

    public void setPassingScore(Float passingScore) {
        this.passingScore = passingScore;
    }

    public User getAuthor() {
        return this.author;
    }

    public void setAuthor(User author) {
        this.author = author;
    }

    public Course getCourse() {
        return this.course;
    }

    public void setCourse(Course course) {
        this.course = course;
    }

    public Area getArea() {
        return this.area;
    }

    public void setArea(Area area) {
        this.area = area;
    }

    public Model getModel() {
        return this.model;
    }

    public void setModel(Model model) {
        this.model = model;
    }

    public Engine getEngine() {
        return this.engine;
    }

    public void setEngine(Engine engine) {
        this.engine = engine;
    }




    @Override
    public int hashCode() {
        return Objects.hashCode(getId());
    }

    @Override
    public String toString() {
        return "{ QuestionPaper: { " +
            "'id': '" + getId() + "'" +
            ", 'importId': '" + getImportId() + "'" +
            ", 'title': '" + getTitle() + "'" +
            ", 'description': '" + getDescription() + "'" +
            ", 'testLength': '" + getTestLength() + "'" +
            ", 'type': '" + getType() + "'" +
            ", 'bIsPartsOfBuoy': '" + getBIsPartsOfBuoy() + "'" +
            ", 'bIsLocationHints': '" + getBIsLocationHints() + "'" +
            ", 'bIsEnvironmentalSound': '" + getBIsEnvironmentalSound() + "'" +
            ", 'bIsOperatingHints': '" + getBIsOperatingHints() + "'" +
            ", 'bIsVoiceHints': '" + getBIsVoiceHints() + "'" +
            ", 'bIsNameTag': '" + getBIsNameTag() + "'" +
            ", 'bIsGraphicPopupWindow': '" + getBIsGraphicPopupWindow() + "'" +
            ", 'bIsWarningNotice': '" + getBIsWarningNotice() + "'" +
            ", 'bIsDelete': '" + getBIsDelete() + "'" +
            ", 'randomAbnormals': '" + getRandomAbnormals() + "'" +
            ", 'exceptionGenerateType': '" + getExceptionGenerateType() + "'" +
            ", 'qualifiedQuantity': '" + getQualifiedQuantity() + "'" +
            ", 'score': '" + getScore() + "'" +
            ", 'passingScore': '" + getPassingScore() + "'" +
            " }}";
    }
}
