package cn.com.sailfish.linghang.domain;

import javax.persistence.*;
import javax.validation.constraints.Size;
import java.io.Serializable;
import java.time.Instant;
import java.util.Objects;
import java.util.List;

import cn.com.sailfish.linghang.common.auditor.Auditable;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

/**
 * @author: AutoCoder (maintained by newma<newma@live.cn>)
 * @create: 2019-07-11 11:19:25
 */

/**
 * 用户信息表
 */
@Entity
@Data
@Table(name = "user")
public class User extends Auditable implements Serializable {

    private static final long serialVersionUID = 1L;

    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    @Column(name = "id")
    private Long id;

    @ApiModelProperty(value="用户名 长度在1到50个字符串之间，不为空，允许重复")
    @Column(name = "username", nullable = false)
    @Size(min = 1, max = 50)
    private String username; // 用户名 长度在1到50个字符串之间，不为空，允许重复

    @ApiModelProperty(value="密码, 长度255, 不为空")
    @Column(name = "password", nullable = false)
    private String password; // 密码, 长度255, 不为空

    @ApiModelProperty(value="用户工号,长度在6到18之间，不为空且唯一")
    @Column(name = "card")
    @Size(min = 6, max = 18)
    private String card; // 用户工号,长度在6到18之间，不为空且唯一

    @ApiModelProperty(value="用户头像")
    @Column(name = "user_image")
    private String userImage; // 用户头像

    @ApiModelProperty(value="性别 0:女; 1:男")
    @Column(name = "gender")
    private Integer gender; // 性别 0:女; 1:男

    @ApiModelProperty(value="用户的手机号码, 长度是11,手机号码不为空且唯一")
    @Column(name = "mobile_phone", length = 11)
    private String mobilePhone; // 用户的手机号码, 长度是11,手机号码不为空且唯一

    @ApiModelProperty(value="用户类型 0 代表学员 1 代表教员")
    @Column(name = "type")
    private Integer type; // 用户类型 0 代表学员 1 代表教员

    @ApiModelProperty(value="班别 0初训 1复训")
    @Column(name = "class_type")
    private String classType; // 班别 0初训 1复训

    @ApiModelProperty(value="备注")
    @Column(name = "remarks")
    private String remarks; // 备注

    @ApiModelProperty(value="用户的邮箱, 长度是50,唯一, 允许为空")
    @Column(name = "email")
    @Size(min = 0, max = 50)
    private String email; // 用户的邮箱, 长度是50,唯一, 允许为空

    @ApiModelProperty(value="上次登录的IP, 允许为空")
    @Column(name = "last_login_ip")
    @Size(min = 0, max = 20)
    private String lastLoginIp; // 上次登录的IP, 允许为空

    @ApiModelProperty(value="上次登录的时间")
    @Column(name = "last_login_time")
    private Instant lastLoginTime; // 上次登录的时间

    @ApiModelProperty(value="是否在考试队列中 true 代表在 false代表不在")
    @Column(name = "b_is_test_queue", nullable = false)
    private Boolean biosIsTestQueue; // 是否在考试队列中 true 代表在 false代表不在

    @ApiModelProperty(value="是否是新添加到考试队列 true 代表是 false代表不是")
    @Column(name = "b_is_new_add_test_queue", nullable = false)
    private Boolean biosIsNewAddTestQueue; // 是否是新添加到考试队列 true 代表是 false代表不是

    @ApiModelProperty(value="用户当前状态：0 训练 1 考试 2 离线 3 在线，默认值为2")
    @Column(name = "status", nullable = false)
    private Integer status; // 用户当前状态：0 训练 1 考试 2 离线 3 在线，默认值为2

    @ApiModelProperty(value="用户激活状态：true 激活 false 未激活（回收站）, 默认值为true")
    @Column(name = "activation", nullable = false)
    private Boolean activation; // 用户激活状态：true 激活 false 未激活（回收站）, 默认值为true

    @ApiModelProperty(value=" 用户的权限：1 开发者权限 2 超级管理员 4 机构管理员 8 教员 16 普通用户, 默认值是16")
    @Column(name = "authority", nullable = false)
    private Integer authority; // 用户的权限：1 开发者权限 2 超级管理员 4 机构管理员 8 教员 16 普通用户, 默认值是16

    @ApiModelProperty(value="最近指定的监考员信息")
    @Column(name = "invigilator_message")
    private String invigilatorMessage; // 最近指定的监考员信息

    @ApiModelProperty(value="登录的VR设备的队列名")
    @Column(name = "queue_name")
    private String queueName; // 登录的VR设备的队列名

    @ApiModelProperty(value="用户是否启用权限：true 启用 false 不启用, 默认值为true")
    @Column(name = "enable", nullable = false)
    private Boolean enable; // 用户是否启用权限：true 启用 false 不启用, 默认值为true

    @ApiModelProperty(value="用户是否被删除")
    @Column(name="is_del")
    private Integer isDel; // 用户是否被删除

    @ApiModelProperty(value="权限控制 前端权限为0 后端为1 都有就0,1 都没就空")
    @Column(name="authority_template")
    private String authorityTemplate; // 权限控制


    @ManyToOne
    @JoinColumn(name = "leader_id")
    private User leader;


    @ManyToOne
    @JoinColumn(name = "area_id")
    private Area area;


    // getter and setter

}
