package cn.com.sailfish.linghang.repository;

import cn.com.sailfish.linghang.domain.Area;
import cn.com.sailfish.linghang.domain.Exam;
import cn.com.sailfish.linghang.domain.QuestionPaper;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.Query;
import org.springframework.stereotype.Repository;

import java.time.Instant;
import java.util.List;

/**
 * Author: AutoCoder (maintained by newma<newma@live.cn>)
 * Create: 2019-04-23 15:12:04
 * @author huangpeilin
 */

@Repository
public interface ExamRepository extends JpaRepository<Exam, Long> {

    /**
     * 通过区域分页获取考试信息
     * @param area 区域
     * @param pageable
     * @return
     */
    Page<Exam> findAllByAreaAndBiosIsDelete(Area area, boolean biosIsDelete, Pageable pageable);

    /**
     * 通过区域和试卷名字搜索考试
     * @param area 区域
     * @param biosIsDelete 是否在回收站中 false 没有 true 是
     * @param questionPaperName 试卷名字
     * @param pageable
     * @return
     */
    @Query("select e from Exam e where e.area = ?1 and e.biosIsDelete = ?2 and e.questionPaper.title like %?3%")
    Page<Exam> findAllByAreaAndBiosIsDeleteAndQuestionPaperTitleLike(Area area, boolean biosIsDelete,
                                                                   String questionPaperName, Pageable pageable);

    /**
     * 通过区域和班别获取考试
     * @param area 区域
     * @param classType 班别
     * @param pageable
     * @return
     */
    @Query(value = "select e from Exam e where e.area = ?1 and e.biosIsDelete = ?2 and e.classType = ?3")
    Page<Exam> findAllByAreaAndBiosIsDeleteAndClassType(Area area, Boolean biosIsDelete, String classType,
                                                      Pageable pageable);

    /**
     * 通过区域，班别和试卷名字获取考试
     * @param area 区域
     * @param biosIsDelete 是否在回收站中 false没有 true 是
     * @param classType 班别
     * @param questionPaperName 试卷名字
     * @param pageable
     * @return
     */
    @Query(value = "select e from Exam e where e.area = ?1 and e.biosIsDelete = ?2  and e.classType = ?3 and e.questionPaper.title like %?4%")
    Page<Exam> findAllByAreaAndBiosIsDeleteAndClassTypeAndQuestionPaperTitleLike(Area area, Boolean biosIsDelete,
                                                                               String classType, String questionPaperName, Pageable pageable);

    /**
     * 根据区域和监考人名字分页获取考试
     * @param area 区域
     * @param biosIsDelete 是否在回收站中 false没有 true 是
     * @param invigilator 监考人名字
     * @param pageable
     * @return
     */
    @Query(value = "select e from Exam e where e.area = ?1 and e.biosIsDelete = ?2  and e.invigilator like %?3%")
    Page<Exam> findAllByAreaAndBiosIsDeleteAndInvigilatorLike(Area area, Boolean biosIsDelete, String invigilator,
                                                            Pageable pageable);

    /**
     * 根据区域，监考人名字和试卷名字分页获取考试信息
     * @param area 区域
     * @param biosIsDelete 是否在回收站中 false没有 true 是
     * @param questionPaperName 试卷名字
     * @param invigilator 监考人名字
     * @param pageable
     * @return
     */
    @Query(value = "select e from Exam e where e.area = ?1 and e.biosIsDelete = ?2  and e.questionPaper.title like %?3% and e.invigilator like %?4% ")
    Page<Exam> findAllByAreaAndBiosIsDeleteAndQuestionPaperTitleLikeAndInvigilatorLike(Area area, Boolean biosIsDelete,
                                                                                     String questionPaperName, String invigilator, Pageable pageable);

    /**
     * 根据区域，监考人名字和班别分页获取考试信息
     * @param area 区域
     * @param biosIsDelete 是否在回收站中 false没有 true 是
     * @param classType 班别
     * @param invigilator 监考人名字
     * @param pageable
     * @return
     */
    @Query(value = "select e from Exam e where e.area = ?1 and e.biosIsDelete = ?2  and e.classType = ?3 and e.invigilator like %?4% ")
    Page<Exam> findAllByAreaAndBiosIsDeleteAndClassTypeAndInvigilatorLike(Area area, Boolean biosIsDelete,
                                                                        String classType, String invigilator, Pageable pageable);

    /**
     * 根据区域，监考人名字，试卷名字和班别分页获取考试信息
     * @param area 区域
     * @param biosIsDelete 是否在回收站中 false没有 true 是
     * @param classType 班别
     * @param questionPaperName 试卷名字
     * @param invigilator 监考人名字
     * @param pageable
     * @return
     */
    @Query(value = "select e from Exam e where e.area = ?1 and e.biosIsDelete = ?2  and e.classType = ?3 and e.questionPaper.title like %?4% and e.invigilator like %?5% ")
    Page<Exam> findAllByAreaAndBiosIsDeleteAndClassTypeAndQuestionPaperTitleLikeAndInvigilatorLike(
            Area area, Boolean biosIsDelete, String classType, String questionPaperName, String invigilator, Pageable pageable);

    /**
     * 根据区域分页获取某段时间考试信息
     * @param area 区域
     * @param biosIsDelete 是否在回收站中 false没有 true 是
     * @param startTime 开始时间
     * @param endTime 结束时间
     * @param pageable
     * @return
     */
    @Query(value = "select e from Exam e where e.area = ?1 and e.biosIsDelete = ?2  and e.examTime between ?3 and ?4")
    Page<Exam> findAllByAreaAndBiosIsDeleteAndExamTimeBetween(Area area, Boolean biosIsDelete, Instant startTime,
                                                            Instant endTime, Pageable pageable);


    /**
     * 根据区域和试卷名字分页获取某段时间考试信息
     * @param area 区域
     * @param biosIsDelete 是否在回收站中 false没有 true 是
     * @param questionPaperName 试卷名字
     * @param startTime 开始时间
     * @param endTime 结束时间
     * @param pageable
     * @return
     */
    @Query(value = "select e from Exam e where e.area = ?1 and e.biosIsDelete = ?2  and e.questionPaper.title like %?3% and e.examTime between ?4 and ?5")
    Page<Exam> findAllByAreaAndBiosIsDeleteAndQuestionPaperTitleLikeAndExamTimeBetween(Area area, Boolean biosIsDelete,
                                                                                     String questionPaperName, Instant startTime, Instant endTime, Pageable pageable);

    /**
     * 根据区域和班别分页获取某段时间考试信息
     * @param area 区域
     * @param biosIsDelete 是否在回收站中 false没有 true 是
     * @param classType 班别
     * @param startTime 开始时间
     * @param endTime 结束时间
     * @param pageable
     * @return
     */
    @Query(value = "select e from Exam e where e.area = ?1 and e.biosIsDelete = ?2  and e.classType = ?3 and e.examTime between ?4 and ?5")
    Page<Exam> findAllByAreaAndBiosIsDeleteAndClassTypeAndExamTimeBetween(Area area, Boolean biosIsDelete,
                                                                        String classType, Instant startTime, Instant endTime, Pageable pageable);

    /**
     * 根据区域，试卷名字和班别分页获取某段时间考试信息
     * @param area 区域
     * @param biosIsDelete 是否在回收站中 false没有 true 是
     * @param classType 班别
     * @param questionPaperName 试卷名字
     * @param startTime 开始时间
     * @param endTime 结束时间
     * @param pageable
     * @return
     */
    @Query(value = "select e from Exam e where e.area = ?1 and e.biosIsDelete = ?2  and e.classType = ?3 and e.questionPaper.title like %?4% and e.examTime between ?5 and ?6")
    Page<Exam> findAllByAreaAndBiosIsDeleteAndClassTypeAndQuestionPaperTitleLikeAndExamTimeBetween(
            Area area, Boolean biosIsDelete, String classType, String questionPaperName, Instant startTime, Instant endTime, Pageable pageable);

    /**
     * 根据区域和监考人名字分页获取某段时间考试信息
     * @param area 区域
     * @param biosIsDelete 是否在回收站中 false没有 true 是
     * @param invigilator 监考人名字
     * @param startTime 开始时间
     * @param endTime 结束时间
     * @param pageable
     * @return
     */
    @Query(value = "select e from Exam e where e.area = ?1 and e.biosIsDelete = ?2  and e.invigilator like %?3% and e.examTime between ?4 and ?5")
    Page<Exam> findAllByAreaAndBiosIsDeleteAndInvigilatorLikeAndExamTimeBetween(Area area, Boolean biosIsDelete,
                                                                              String invigilator, Instant startTime, Instant endTime, Pageable pageable);


    /**
     * 根据区域，试卷名字和监考人名字分页获取某段时间考试信息
     * @param area 区域
     * @param biosIsDelete 是否在回收站中 false没有 true 是
     * @param questionPaperName 试卷名字
     * @param invigilator 监考人名字
     * @param startTime 开始时间
     * @param endTime 结束时间
     * @param pageable
     * @return
     */
    @Query(value = "select e from Exam e where e.area = ?1 and e.biosIsDelete = ?2  and e.questionPaper.title like %?3% and e.invigilator like %?4% and e.examTime between ?5 and ?6")
    Page<Exam> findAllByAreaAndBiosIsDeleteAndQuestionPaperTitleLikeAndInvigilatorLikeAndExamTimeBetween(
            Area area, Boolean biosIsDelete, String questionPaperName, String invigilator, Instant startTime, Instant endTime, Pageable pageable);

    /**
     * 根据区域，班别和监考人名字分页获取某段时间考试信息
     * @param area 区域
     * @param biosIsDelete 是否在回收站中 false没有 true 是
     * @param invigilator 监考人名字
     * @param classType 班别
     * @param startTime 开始时间
     * @param endTime 结束时间
     * @param pageable
     * @return
     */
    @Query(value = "select e from Exam e where e.area = ?1 and e.biosIsDelete = ?2  and e.classType = ?3 and e.invigilator like %?4% and e.examTime between ?5 and ?6")
    Page<Exam> findAllByAreaAndBiosIsDeleteAndClassTypeAndInvigilatorLikeAndExamTimeBetween(
            Area area, Boolean biosIsDelete, String classType, String invigilator, Instant startTime, Instant endTime, Pageable pageable);

    /**
     * 根据区域，试卷名字和监考人名字分页获取某段时间考试信息
     * @param area 区域
     * @param biosIsDelete 是否在回收站中 false没有 true 是
     * @param classType 班别
     * @param questionPaperName 试卷名字
     * @param invigilator 监考人名字
     * @param startTime 开始时间
     * @param endTime 结束时间
     * @param pageable
     * @return
     */
    @Query(value = "select e from Exam e where e.area = ?1 and e.biosIsDelete = ?2  and e.classType = ?3 and e.questionPaper.title like %?4% and e.invigilator like %?5% and e.examTime between ?6 and ?7")
    Page<Exam> findAllByAreaAndBiosIsDeleteAndClassTypeAndQuestionPaperTitleLikeAndInvigilatorLikeAndExamTimeBetween(
            Area area, Boolean biosIsDelete, String classType, String questionPaperName, String invigilator, Instant startTime, Instant endTime, Pageable pageable);

    /**
     * 通过考试ID查找考试
     * @param id
     * @return
     */
    Exam findOneById(Long id);

    List<Exam> findAllByQuestionPaper(QuestionPaper questionPaper);

    /**
     * 获取某个区域的所有考试
     * @param area
     * @return
     */
    List<Exam> findAllByArea(Area area);
}