package cn.com.sailfish.linghang.repository;

import cn.com.sailfish.linghang.domain.Area;
import cn.com.sailfish.linghang.domain.OperationRecord;
import cn.com.sailfish.linghang.domain.User;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.Query;
import org.springframework.stereotype.Repository;
import org.springframework.web.bind.annotation.RequestParam;

import java.time.Instant;
import java.util.List;

/**
 * Author: AutoCoder (maintained by newma<newma@live.cn>)
 * Create: 2019-04-23 15:12:04
 *
 * @author huangpeilin
 */

@Repository
public interface OperationRecordRepository extends JpaRepository<OperationRecord, Long> {

    /**
     * 根据操作类型获取操作记录
     * @param area 区域
     * @param type 操作类型
     * @param status 是否是最新状态 0 是 1 不是
     * @param pageable
     * @return
     */
    Page<OperationRecord> findAllByAreaAndTypeAndStatus(Area area, Integer type, Integer status, Pageable pageable);

    /**
     * 根据操作类型和用户名获取操作记录
     * @param area 区域
     * @param type 操作类型
     * @param status 是否是最新状态 0 是 1 不是
     * @param username 用户名
     * @param pageable
     * @return
     */
    @Query("select o from OperationRecord o where o.area = ?1 and o.type = ?2 and o.status = ?3 and o.username like %?4%")
    Page<OperationRecord> findAllByAreaAndTypeAndStatusAndUsernameLike(Area area, Integer type, Integer status, String username, Pageable pageable);

    /**
     * 根据操作类型和工号获取操作记录
     * @param area 区域
     * @param type 操作类型
     * @param status 是否是最新状态 0 是 1 不是
     * @param card 工号
     * @param pageable
     * @return
     */
    @Query("select o from OperationRecord o where o.area = ?1 and o.type = ?2 and o.status = ?3 and o.card like %?4%")
    Page<OperationRecord> findAllByAreaAndTypeAndStatusAndCardLike(Area area, Integer type, Integer status, String card, Pageable pageable);

    /**
     * 根据操作类型,用户名和工号获取操作记录
     * @param area 区域
     * @param type 操作类型
     * @param status 是否是最新状态 0 是 1 不是
     * @param username 用户名字
     * @param card 用户工号
     * @param pageable
     * @return
     */
    @Query("select o from OperationRecord o where o.area = ?1 and o.type = ?2 and o.status = ?3 and o.username like %?4% and o.card like %?5%")
    Page<OperationRecord> findAllByAreaAndTypeAndStatusAndUsernameLikeAndCardLike(Area area, Integer type, Integer status, String username, String card, Pageable pageable);

    /**
     * 根据操作类型和试卷名字（课程名字）获取操作记录
     * @param area 区域
     * @param type 操作类型
     * @param status 是否是最新状态 0 是 1 不是
     * @param typeName 试卷名字（课程名字）
     * @param pageable
     * @return
     */
    @Query("select o from OperationRecord o where o.area = ?1 and o.type = ?2 and o.status = ?3 and o.typeName like %?4%")
    Page<OperationRecord> findAllByAreaAndTypeAndStatusAndTypeNameLike(Area area, Integer type, Integer status, String typeName, Pageable pageable);

    /**
     * 根据操作类型，试卷名字（课程名字）和用户名获取操作记录
     * @param area 区域
     * @param type 操作类型
     * @param status 是否是最新状态 0 是 1 不是
     * @param typeName 试卷名字（课程名字）
     * @param username 用户名
     * @param pageable
     * @return
     */
    @Query("select o from OperationRecord o where o.area = ?1 and o.type = ?2 and o.status = ?3 and o.typeName like %?4% and o.username like %?5%")
    Page<OperationRecord> findAllByAreaAndTypeAndStatusAndTypeNameLikeAndUsernameLike(Area area, Integer type, Integer status, String typeName, String username, Pageable pageable);

    /**
     * 根据操作类型，试卷名字（课程名字）和用户工号获取操作记录
     * @param area 区域
     * @param type 操作类型
     * @param status 是否是最新状态 0 是 1 不是
     * @param card 用户工号
     * @param name 试卷名字（课程名字）
     * @param pageable
     * @return
     */
    @Query("select o from OperationRecord o where o.area = ?1 and o.type = ?2 and o.status = ?3 and o.card like %?4% and o.typeName like %?5%")
    Page<OperationRecord> findAllByAreaAndTypeAndStatusAndCardLikeAndTypeNameLike(Area area, Integer type, Integer status, String card, String name, Pageable pageable);

    /**
     * 根据操作类型，试卷名字（课程名字）、用户名字和用户工号获取操作记录
     * @param area 区域
     * @param type 操作类型
     * @param status 是否是最新状态 0 是 1 不是
     * @param username 用户名
     * @param card 用户工号
     * @param name 试卷名字（课程名字）
     * @param pageable
     * @return
     */
    @Query("select o from OperationRecord o where o.area = ?1 and o.type = ?2 and o.status = ?3 and o.username like %?4% and o.card like %?5% and o.typeName like %?6%")
    Page<OperationRecord> findAllByAreaAndTypeAndStatusAndUsernameLikeAndCardLikeAndTypeNameLike(Area area, Integer type, Integer status, String username, String card, String name, Pageable pageable);

    /**
     * 根据操作类型获取某个时间端内的操作记录
     * @param area 区域
     * @param type 操作类型
     * @param status 是否是最新状态 0 是 1 不是
     * @param startTime 开始时间
     * @param endTime 结束时间
     * @param pageable
     * @return
     */
    @Query("select o from OperationRecord o where o.area = ?1 and o.type = ?2 and o.status = ?3 and o.loginTime between ?4 and ?5")
    Page<OperationRecord> findAllByAreaAndTypeAndStatusAndLoginTimeBetween(Area area,Integer type, Integer status, Instant startTime, Instant endTime, Pageable pageable);

    /**
     * 根据操作类型和用户名字获取某个时间端内的操作记录
     * @param area 区域
     * @param type 操作类型
     * @param status 是否是最新状态 0 是 1 不是
     * @param username 用户名字
     * @param startTime 开始时间
     * @param endTime 结束时间
     * @param pageable
     * @return
     */
    @Query("select o from OperationRecord o where o.area = ?1 and o.type = ?2 and o.status = ?3 and o.username like %?4% and o.loginTime between ?5 and ?6")
    Page<OperationRecord> findAllByAreaAndTypeAndStatusAndUsernameLikeAndLoginTimeBetween(Area area, Integer type, Integer status, String username, Instant startTime, Instant endTime, Pageable pageable);

    /**
     * 根据操作类型和用户卡号获取某个时间端内的操作记录
     * @param area 区域
     * @param type 操作类型
     * @param status 是否是最新状态 0 是 1 不是
     * @param card 用户工卡
     * @param startTime 开始时间
     * @param endTime 结束时间
     * @param pageable
     * @return
     */
    @Query("select o from OperationRecord o where o.area = ?1 and o.type = ?2 and o.status = ?3 and o.card like %?4% and o.loginTime between ?5 and ?6")
    Page<OperationRecord> findAllByAreaAndTypeAndStatusAndCardLikeAndLoginTimeBetween(Area area, Integer type, Integer status, String card, Instant startTime, Instant endTime, Pageable pageable);

    /**
     * 根据操作类型、用户卡号和用户名字获取某个时间端内的操作记录
     * @param area 区域
     * @param type 操作类型
     * @param status 是否是最新状态 0 是 1 不是
     * @param username 用户名字
     * @param card 用户工卡
     * @param startTime 开始时间
     * @param endTime 结束时间
     * @param pageable
     * @return
     */
    @Query("select o from OperationRecord o where o.area = ?1 and o.type = ?2 and o.status = ?3 and o.username like %?4% and o.card like %?5% and o.loginTime between ?6 and ?7")
    Page<OperationRecord> findAllByAreaAndTypeAndStatusAndUsernameLikeAndCardLikeAndLoginTimeBetween(Area area, Integer type, Integer status, String username, String card, Instant startTime, Instant endTime, Pageable pageable);

    /**
     * 根据操作类型和试卷名字（课程名字）获取某个时间端内的操作记录
     * @param area 区域
     * @param type 操作类型
     * @param status 是否是最新状态 0 是 1 不是
     * @param name 试卷名字（课程名字）
     * @param startTime 开始时间
     * @param endTime 结束时间
     * @param pageable
     * @return
     */
    @Query("select o from OperationRecord o where o.area = ?1 and o.type = ?2 and o.status = ?3 and o.typeName like %?4% and o.loginTime between ?5 and ?6")
    Page<OperationRecord> findAllByAreaAndTypeAndStatusAndTypeNameLikeAndLoginTimeBetween(Area area, Integer type, Integer status, String name, Instant startTime, Instant endTime, Pageable pageable);

    /**
     * 根据操作类型、试卷名字（课程名字）和用户名字获取某个时间端内的操作记录
     * @param area 区域
     * @param type 操作类型
     * @param status 是否是最新状态 0 是 1 不是
     * @param name 试卷名字（课程名字）
     * @param username 用户名字
     * @param startTime 开始时间
     * @param endTime 结束时间
     * @param pageable
     * @return
     */
    @Query("select o from OperationRecord o where o.area = ?1 and o.type = ?2 and o.status = ?3 and o.typeName like %?4% and o.username like %?5% and o.loginTime between ?6 and ?7")
    Page<OperationRecord> findAllByAreaAndTypeAndStatusAndTypeNameLikeAndUsernameLikeAndLoginTimeBetween(Area area, Integer type, Integer status, String name, String username, Instant startTime, Instant endTime, Pageable pageable);

    /**
     * 根据操作类型、试卷名字（课程名字）和用户卡号获取某个时间端内的操作记录
     * @param area 区域
     * @param type 操作类型
     * @param status 是否是最新状态 0 是 1 不是
     * @param card 用户卡号
     * @param name 试卷名字（课程名字）
     * @param startTime 开始时间
     * @param endTime 结束时间
     * @param pageable
     * @return
     */
    @Query("select o from OperationRecord o where o.area = ?1 and o.type = ?2 and o.status = ?3 and o.card like %?4% and o.typeName like %?5% and o.loginTime between ?6 and ?7")
    Page<OperationRecord> findAllByAreaAndTypeAndStatusAndCardLikeAndTypeNameLikeAndLoginTimeBetween(Area area, Integer type, Integer status, String card, String name, Instant startTime, Instant endTime, Pageable pageable);

    /**
     * 根据操作类型、试卷名字（课程名字）、用户名字和用户卡号获取某个时间端内的操作记录
     * @param area 区域
     * @param type 操作类型
     * @param status 是否是最新状态 0 是 1 不是
     * @param username 用户名字
     * @param card 用户卡号
     * @param name 试卷名字（课程名字）
     * @param startTime 开始时间
     * @param endTime 结束时间
     * @param pageable
     * @return
     */
    @Query("select o from OperationRecord o where o.area = ?1 and o.type = ?2 and o.status = ?3 and o.username like %?4% and o.card like %?5% and o.typeName like %?6% and o.loginTime between ?7 and ?8")
    Page<OperationRecord> findAllByAreaAndTypeAndStatusAndUsernameLikeAndCardLikeAndTypeNameLikeAndLoginTimeBetween(Area area, Integer type, Integer status, String username, String card, String name, Instant startTime, Instant endTime, Pageable pageable);

    Page<OperationRecord> findAllByAreaAndStatus(Area area, Integer status, Pageable pageable);

    /**
     * 根据用户卡号和操作类型获取最新的操作记录
     * @param card 用户卡号
     * @param status 是否是最新状态 0 是 1 不是
     * @param type 操作类型
     * @return
     */
    OperationRecord findOneByCardAndStatusAndType(String card, Integer status, Integer type);

    /**
     * 根据区域和操作类型获取操作记录
     * @param area 区域
     * @param type 操作类型
     * @param pageable
     * @return
     */
    Page<OperationRecord> findAllByAreaAndType(Area area, Integer type, Pageable pageable);

    /**
     * 根据操作类型和状态获取某个区域的操作记录
     * @param area 区域
     * @param type 操作类型
     * @param status 是否是最新状态 0 是 1 不是
     * @return
     */
    List<OperationRecord> findAllByAreaAndTypeAndStatus(Area area, Integer type, Integer status);

    /**
     * 根据用户卡号和操作类型获取操作记录
     * @param type 操作类型
     * @param card 用户卡号
     * @return
     */
    @Query("select o from OperationRecord o where o.type = ?1 and o.card = ?2 and (o.courseId is not null or o.examId is not null )")
    List<OperationRecord> findAllByTypeAndCard(Integer type, String card);

    /**
     * 根据卡号获取登出时间为空的操作记录
     * @param card
     * @return
     */
    List<OperationRecord> findAllByCardAndLogoutTimeIsNull(String card);

    /**
     * 根据卡号，操作类型和登录时间获取操作记录
     * @param card 用户工卡
     * @param type 操作类型
     * @param loginTime 登录时间
     * @return
     */
    List<OperationRecord> findAllByCardAndTypeAndLoginTime(String card, Integer type, Instant loginTime);

    /**
     * 根据卡号和操作类型获取操作记录
     * @param area
     * @param type
     * @param card
     * @param pageable
     * @return
     */
    @Query("select o from OperationRecord o where o.area = ?1 and o.type = ?2 and o.card = ?3 and (o.courseId is not null or o.examId is not null )")
    Page<OperationRecord> findAllByAreaAndTypeAndCard(Area area, Integer type, String card, Pageable pageable);
}