package cn.com.sailfish.linghang.repository;

import cn.com.sailfish.linghang.domain.Area;
import cn.com.sailfish.linghang.domain.QuestionPaper;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.Query;
import org.springframework.stereotype.Repository;

/**
 * Author: AutoCoder (maintained by newma<newma@live.cn>)
 * Create: 2019-04-23 15:12:04
 *
 * @author huangpeilin
 */

@Repository
public interface QuestionPaperRepository extends JpaRepository<QuestionPaper, Long> {

    /**
     * 通过试卷标题和试卷备注查找试卷
     * @param title
     * @param description
     * @return
     */
    QuestionPaper findOneByTitleAndDescription(String title, String description);

    /**
     * 通过区域查找试卷
     * @param area 区域
     * @param bIsDelete 是否搜索回收站的试卷 true 是 false 不是
     * @param pageable
     * @return
     */
    Page<QuestionPaper> findAllByAreaAndBIsDelete(Area area, Boolean bIsDelete, Pageable pageable);

    /**
     * 通过试卷ID获取试卷
     * @param id
     * @return
     */
    QuestionPaper findOneById(Long id);

    /**
     * 根据区域和创建人名字搜索试卷
     * @param area 区域
     * @param bIsDelete 是否在回收站中 false代表不是 true代表是
     * @param authorName 创建人名字 模糊
     * @param pageable
     * @return
     */
    @Query("select q from QuestionPaper q where q.area = ?1 and q.bIsDelete = ?2 and q.author.username like %?3%")
    Page<QuestionPaper> findAllByAreaAndBIsDeleteAndAuthorUsernameLike(Area area, boolean bIsDelete, String authorName, Pageable pageable);

    /**
     * 根据区域和试卷名字搜索试卷
     * @param area 区域
     * @param bIsDelete 是否在回收站中 false代表不是 true代表是
     * @param title 试卷名字 模糊
     * @param pageable
     * @return
     */
    @Query("select q from QuestionPaper q where q.area = ?1 and q.bIsDelete = ?2 and q.title like %?3%")
    Page<QuestionPaper> findAllByAreaAndBIsDeleteAndTitleLike(Area area, boolean bIsDelete, String title, Pageable pageable);

    /**
     * 根据区域和试卷名字搜索试卷
     * @param area 区域
     * @param bIsDelete 是否在回收站中 false代表不是 true代表是
     * @param title 试卷名字 模糊
     * @param authorName 创建人名字 模糊
     * @param pageable
     * @return
     */
    @Query("select q from QuestionPaper q where q.area = ?1 and q.bIsDelete = ?2 and q.title like %?3% and q.author.username like %?4%")
    Page<QuestionPaper> findAllByAreaAndBIsDeleteAndTitleLikeAndAuthorUsernameLike(Area area, boolean bIsDelete, String title, String authorName, Pageable pageable);

    /**
     * 根据区域和机型ID查找试卷
     * @param area 区域
     * @param bIsDelete 是否在回收站中 false代表不是 true代表是
     * @param modelId 机型ID
     * @param pageable
     * @return
     */
    @Query("select q from QuestionPaper q where q.area = ?1 and q.bIsDelete = ?2 and q.model.id =?3")
    Page<QuestionPaper> findAllByAreaAndBIsDeleteAndModelId(Area area, boolean bIsDelete, Long modelId, Pageable pageable);

    /**
     * 根据区域，机型ID和创建人名字查找试卷
     * @param area 区域
     * @param bIsDelete 是否在回收站中 false代表不是 true代表是
     * @param modelId 机型ID
     * @param authorName 创建人名字 模糊
     * @param pageable
     * @return
     */
    @Query("select q from QuestionPaper q where q.area = ?1 and q.bIsDelete = ?2 and q.model.id = ?3 and q.author.username like %?4%")
    Page<QuestionPaper> findAllByAreaAndBIsDeleteAndModelIdAndAuthorUsernameLike(Area area, boolean bIsDelete, Long modelId, String authorName, Pageable pageable);

    /**
     * 根据区域，课程ID和创建人名字查找试卷
     * @param area 区域
     * @param bIsDelete 是否在回收站中 false代表不是 true代表是
     * @param courseId 课程ID
     * @param pageable
     * @return
     */
    @Query("select q from QuestionPaper q where q.area = ?1 and q.bIsDelete = ?2 and q.course.id = ?3")
    Page<QuestionPaper> findAllByAreaAndBIsDeleteAndCourseId(Area area, boolean bIsDelete, Long courseId, Pageable pageable);

    /**
     * 根据区域，课程ID和创建人名字查找试卷
     * @param area 区域
     * @param bIsDelete 是否在回收站中 false代表不是 true代表是
     * @param courseId 课程ID
     * @param authorName 创建人名字 模糊
     * @param pageable
     * @return
     */
    @Query("select q from QuestionPaper q where q.area = ?1 and q.bIsDelete = ?2 and q.course.id = ?3 and q.author.username like %?4%")
    Page<QuestionPaper> findAllByAreaAndBIsDeleteAndCourseIdAndAuthorUsernameLike(Area area, boolean bIsDelete, Long courseId, String authorName, Pageable pageable);

    /**
     * 根据区域，课程ID和试卷名字查找试卷
     * @param area 区域
     * @param bIsDelete 是否在回收站中 false代表不是 true代表是
     * @param courseId 课程ID
     * @param title 试卷名字 模糊
     * @param pageable
     * @return
     */
    @Query("select q from QuestionPaper q where q.area = ?1 and q.bIsDelete = ?2 and q.course.id = ?3 and q.title like %?4%")
    Page<QuestionPaper> findAllByAreaAndBIsDeleteAndCourseIdAndTitleLike(Area area, boolean bIsDelete, Long courseId, String title, Pageable pageable);

    /**
     * 根据区域，课程ID和试卷名字查找试卷
     * @param area 区域
     * @param bIsDelete 是否在回收站中 false代表不是 true代表是
     * @param courseId 课程ID
     * @param title 试卷名字 模糊
     * @param authorName 创建人名字 模糊
     * @param pageable
     * @return
     */
    @Query("select q from QuestionPaper q where q.area = ?1 and q.bIsDelete = ?2 and q.course.id = ?3 and q.title like %?4% and q.author.username like %?5%")
    Page<QuestionPaper> findAllByAreaAndBIsDeleteAndCourseIdAndTitleLikeAndAuthorUsernameLike(Area area, boolean bIsDelete, Long courseId, String title, String authorName, Pageable pageable);

    /**
     * 根据区域，机型ID和试卷名字查找试卷
     * @param area 区域
     * @param bIsDelete 是否在回收站中 false代表不是 true代表是
     * @param modelId 机型ID
     * @param title 试卷名字 模糊
     * @param pageable
     * @return
     */
    @Query("select q from QuestionPaper q where q.area = ?1 and q.bIsDelete = ?2 and q.model.id = ?3 and q.title like %?4%")
    Page<QuestionPaper> findAllByAreaAndBIsDeleteAndModelIdAndTitleLike(Area area, boolean bIsDelete, Long modelId, String title, Pageable pageable);

    /**
     * 根据区域，机型ID和试卷名字查找试卷
     * @param area 区域
     * @param bIsDelete 是否在回收站中 false代表不是 true代表是
     * @param modelId 机型ID
     * @param title 试卷名字 模糊
     * @param authorName 创建人名字 模糊
     * @param pageable
     * @return
     */
    @Query("select q from QuestionPaper q where q.area = ?1 and q.bIsDelete = ?2 and q.model.id = ?3 and q.title like %?4% and q.author.username like %?5%")
    Page<QuestionPaper> findAllByAreaAndBIsDeleteAndModelIdAndTitleLikeAndAuthorUsernameLike(Area area, boolean bIsDelete, Long modelId, String title, String authorName, Pageable pageable);

    /**
     * 根据区域，机型ID和试卷名字查找试卷
     * @param area 区域
     * @param bIsDelete 是否在回收站中 false代表不是 true代表是
     * @param courseId 课程ID
     * @param modelId 机型ID
     * @param pageable
     * @return
     */
    @Query("select q from QuestionPaper q where q.area = ?1 and q.bIsDelete = ?2 and q.course.id = ?3 and q.model.id = ?4")
    Page<QuestionPaper> findAllByAreaAndBIsDeleteAndCourseIdAndModelId(Area area, boolean bIsDelete, Long courseId, Long modelId, Pageable pageable);

    /**
     * 根据区域，课程ID，机型ID和创建人名字查找试卷
     * @param area 区域
     * @param bIsDelete 是否在回收站中 false代表不是 true代表是
     * @param courseId 课程ID
     * @param modelId 机型ID
     * @param authorName 创建人名字 模糊
     * @param pageable
     * @return
     */
    @Query("select q from QuestionPaper q where q.area = ?1 and q.bIsDelete = ?2 and q.course.id = ?3 and q.model.id = ?4 and q.author.username like %?5%")
    Page<QuestionPaper> findAllByAreaAndBIsDeleteAndCourseIdAndModelIdAndAuthorUsernameLike(Area area, boolean bIsDelete, Long courseId, Long modelId, String authorName, Pageable pageable);

    /**
     * 根据区域，课程ID，机型ID和试卷名字查找试卷
     * @param area 区域
     * @param bIsDelete 是否在回收站中 false代表不是 true代表是
     * @param courseId 课程ID
     * @param modelId 机型ID
     * @param title 试卷名字 模糊
     * @param pageable
     * @return
     */
    @Query("select q from QuestionPaper q where q.area = ?1 and q.bIsDelete = ?2 and q.course.id = ?3 and q.model.id = ?4 and q.title like %?5%")
    Page<QuestionPaper> findAllByAreaAndBIsDeleteAndCourseIdAndModelIdAndTitleLike(Area area, boolean bIsDelete, Long courseId, Long modelId, String title, Pageable pageable);

    /**
     * 根据区域，课程ID，机型ID，试卷名字和创建人名字查找试卷
     * @param area 区域
     * @param bIsDelete 是否在回收站中 false代表不是 true代表是
     * @param courseId 课程ID
     * @param modelId 机型ID
     * @param title 试卷名字 模糊
     * @param authorName 创建人名字 模糊
     * @param pageable
     * @return
     */
    @Query("select q from QuestionPaper q where q.area = ?1 and q.bIsDelete = ?2 and q.course.id = ?3 and q.model.id = ?4 and q.title like %?5% and q.author.username like %?6%")
    Page<QuestionPaper> findAllByAreaAndBIsDeleteAndCourseIdAndModelIdAndTitleLikeAndAuthorUsernameLike(Area area, boolean bIsDelete, Long courseId, Long modelId, String title, String authorName, Pageable pageable);
}