package cn.com.sailfish.linghang.repository;

import cn.com.sailfish.linghang.dao.ReportCardRank;
import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.Query;
import org.springframework.stereotype.Repository;

import java.util.List;

/**
 * Author: huangpeilin
 * Create at: 2019-05-08 16:22:22
 * Description:
 *
 * @author huangpeilin
 */
@Repository
public interface ReportCardRankRepository extends JpaRepository<ReportCardRank, Long> {

    /**
     * 根据考试ID，成绩降序查找班级成绩单
     * @param examId
     * @param page
     * @param pageSize
     * @return
     */
    @Query(value = "select r.*," +
            "(SELECT COUNT(score) FROM report_card rs WHERE rs.exam_id = ?1 and rs.switch_status = 1 and rs.completion_status = 0 and rs.b_is_delete = FALSE and r.score < rs.score) + 1 AS rank " +
            "FROM report_card r WHERE r.exam_id = ?1 and r.switch_status = 1 and r.completion_status = 0 and r.b_is_delete = FALSE " +
            "order by r.score DESC limit ?2,?3", nativeQuery = true)
    List<ReportCardRank> findAllByExamIdAndScoreDesc(Long examId, Integer page, Integer pageSize);

    /**
     * 根据考试ID，工号降序查找班级成绩单
     * @param examId
     * @param page
     * @param pageSize
     * @return
     */
    @Query(value = "select r.*," +
            "(SELECT COUNT(score) FROM report_card rs WHERE rs.exam_id = ?1 and rs.switch_status = 1 and rs.completion_status = 0 and rs.b_is_delete = FALSE and r.score < rs.score) + 1 AS rank " +
            "FROM report_card r WHERE r.exam_id = ?1 and r.switch_status = 1 and r.completion_status = 0 and r.b_is_delete = FALSE " +
            "order by r.card DESC limit ?2,?3", nativeQuery = true)
    List<ReportCardRank> findAllByExamIdAndCardDesc(Long examId, Integer page, Integer pageSize);

    /**
     * 获取班级成绩单的总数
     * @param examId
     * @return
     */
    @Query(value = "select COUNT(*) FROM report_card r WHERE r.exam_id = ?1", nativeQuery = true)
    Integer findAllByExamId(Long examId);

    /**
     * 根据考试ID，工号降序查找班级成绩单
     * @param examId
     * @return
     */
    @Query(value = "select r.*," +
            "(SELECT COUNT(score) FROM report_card rs WHERE rs.exam_id = ?1 and rs.switch_status = 1 and rs.completion_status = 0 and rs.b_is_delete = FALSE and r.score < rs.score) + 1 AS rank " +
            "FROM report_card r WHERE r.exam_id = ?1 and r.switch_status = 1 and r.completion_status = 0 and r.b_is_delete = FALSE " +
            "order by r.card DESC", nativeQuery = true)
    List<ReportCardRank> findAllByExamIdAndCardDesc(Long examId);
}
