package cn.com.sailfish.linghang.service;

import cn.com.sailfish.linghang.domain.Engine;
import cn.com.sailfish.linghang.domain.Model;
import cn.com.sailfish.linghang.exception.ServerErrorException;
import cn.com.sailfish.linghang.repository.ModelRepository;
import cn.com.sailfish.linghang.util.ListUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.List;

import static cn.com.sailfish.linghang.common.ErrorConstants.ENGINE_IS_NOT_EXISTS;
import static cn.com.sailfish.linghang.common.ErrorConstants.MODEL_IS_NOT_EXISTS;

/**
 * Author: huangpeilin
 * Create at: 2019-04-25 10:45:03
 * Description:
 *
 * @author huangpeilin
 */
@Service
public class ModelService {

    public final Logger log = LoggerFactory.getLogger(ModelService.class);

    private ModelRepository modelRepository;

    public ModelService(ModelRepository modelRepository){
        this.modelRepository = modelRepository;
    }

    /**
     * 获取所有的机型品牌
     * @return
     */
    public List<String> findAllBrand() {
        List<String> brandList = new ArrayList<>();
        List<Model> modelList = modelRepository.findAll();
        if (!CollectionUtils.isEmpty(modelList)){
            for (Model model : modelList){
                brandList.add(model.getBrand());
            }
            // 去除重复的数据
            brandList = ListUtil.removeDuplicateWithOrder(brandList);
        }
        return brandList;
    }

    /**
     * 根据机型品牌获取所有机型
     * @param brand 品牌
     * @return
     */
    public List<Model> findAllModelByBrand(String brand) {
        return modelRepository.findAllByBrand(brand);
    }

    /**
     * 获取所有机型
     * @return
     */
    public List<Model> findAllModel() {
        return modelRepository.findAll();
    }

    /**
     * 判断通过机型ID是否可以找到机型
     * @param id
     * @return
     * @throws ServerErrorException
     */
    public Model confirmModelIsExist(Long id) throws ServerErrorException {
        Model model = modelRepository.findOneById(id);
        if (model == null){
            log.error("model is not exist");
            throw new ServerErrorException(MODEL_IS_NOT_EXISTS);
        }
        return model;
    }
}
