package cn.com.sailfish.linghang.service.admin;

import cn.com.sailfish.linghang.dao.web.response.PageResponse;
import cn.com.sailfish.linghang.domain.CourseIntroduce;
import cn.com.sailfish.linghang.domain.OperateDto.CourseIntroduceOperateDto;
import cn.com.sailfish.linghang.domain.OperateDto.ListDto;
import cn.com.sailfish.linghang.repository.CourseIntroduceRepository;
import io.micrometer.core.instrument.util.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.*;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Optional;

@Service
public class CourseIntroduceAdminService {

    @Autowired
    private CourseIntroduceRepository courseIntroduceRepository;

    public PageResponse<CourseIntroduce> findAllByStatue(Integer page, Integer pageSize, String crux) {
        CourseIntroduce courseIntroduce = new CourseIntroduce();
        courseIntroduce.setIsDel(0);

        List<Sort.Order> orders = new ArrayList<>();
        //todo 排序 1发布 2强推 3二级 4 权重 5时间
        orders.add(new Sort.Order(Sort.Direction.DESC, "releaseStatue"));
        orders.add(new Sort.Order(Sort.Direction.DESC, "pushIndex"));
        orders.add(new Sort.Order(Sort.Direction.DESC, "secondTop"));
        orders.add(new Sort.Order(Sort.Direction.ASC, "seqencing"));
        orders.add(new Sort.Order(Sort.Direction.DESC, "gmtCreated"));

        if (!StringUtils.isEmpty(crux)) {
            courseIntroduce.setCourseTitle(crux);
            Example<CourseIntroduce> example = Example.of(courseIntroduce, ExampleMatcher.matching().withMatcher("courseTitle", ExampleMatcher.GenericPropertyMatchers.contains()));
            Pageable pageable = PageRequest.of(page - 1, pageSize, new Sort(orders));
            Page<CourseIntroduce> Courseintroduce = courseIntroduceRepository.findAll(example, pageable);
            return courseIntroducePageResponse(Courseintroduce);
        }

        Example<CourseIntroduce> example = Example.of(courseIntroduce);
        Pageable pageable = PageRequest.of(page - 1, pageSize, new Sort(orders));
        Page<CourseIntroduce> Courseintroduce = courseIntroduceRepository.findAll(example, pageable);
        return courseIntroducePageResponse(Courseintroduce);
    }

    /**
     * description 前移方法
     *
     * @param id
     * @return
     */
    public void forward(long id) {
        CourseIntroduce newspaper = courseIntroduceRepository.findByIdAndSeqencing(id);
        if (newspaper != null) {
            courseIntroduceRepository.forward(id);
        }
    }

    /**
     * description 后移方法
     *
     * @param id
     * @return
     */
    public void backward(long id) {
        courseIntroduceRepository.backward(id);
    }

    /**
     * @param id
     * @return
     * @description 新闻发布；
     */
    public void checkboxrelease(long id) {
        CourseIntroduce courseIntroduce = courseIntroduceRepository.findById(id).orElse(null);
        if (courseIntroduce != null) {
            courseIntroduce.setReleaseStatue(1);
        }

        courseIntroduceRepository.save(courseIntroduce);
    }

    /**
     * @param id
     * @return
     * @description 取消发布
     */
    public void checkboxnorelease(long id) {
        CourseIntroduce courseIntroduce = courseIntroduceRepository.findById(id).orElse(null);
        if (courseIntroduce != null) {
            courseIntroduce.setReleaseStatue(0);
        }

        courseIntroduceRepository.save(courseIntroduce);
    }

    /**
     * @param id
     * @return
     * @description 根据有效状态查询ID；
     */
    public CourseIntroduce findbyid(String id) {
        CourseIntroduce courseIntroduce = courseIntroduceRepository.findCourseIntroducesById(Long.valueOf(id));
        return courseIntroduce;
    }

    public synchronized PageResponse<CourseIntroduce> courseIntroducePageResponse(Page<CourseIntroduce> newscourseIntroduce) {
        PageResponse pageResponse = new PageResponse();
        if (newscourseIntroduce != null) {
            pageResponse.setPageIndex(newscourseIntroduce.getPageable().getPageNumber() + 1);
            pageResponse.setPageSize(newscourseIntroduce.getPageable().getPageSize());
            pageResponse.setTotalCount(newscourseIntroduce.getTotalElements());
            pageResponse.setTotalPageCount(newscourseIntroduce.getTotalPages());
            pageResponse.setList(newscourseIntroduce.getContent());
        }

        return pageResponse;
    }

    public void changePushIndex(String courseId) {
        List<CourseIntroduce> courseByPushIndex = courseIntroduceRepository.findCourseByPushIndex();

        CourseIntroduce courseIntroduce = courseIntroduceRepository.findCourseIntroducesById(Long.valueOf(courseId));
        if (courseIntroduce != null) {
            if (courseIntroduce.getPushIndex() == 0) {
                if(courseByPushIndex.size() >=4 ) {
                    throw new RuntimeException("强推不能拥有大于4条记录");
                }
                courseIntroduce.setPushIndex(1);
            } else {
                courseIntroduce.setPushIndex(0);
            }
            courseIntroduceRepository.save(courseIntroduce);
        }
    }

    public void changeSecondTop(String courseId) {
        CourseIntroduce courseIntroduce = courseIntroduceRepository.findCourseIntroducesById(Long.valueOf(courseId));
        if (courseIntroduce != null) {
            if (courseIntroduce.getSecondTop() == 0) {
                courseIntroduce.setSecondTop(1);
            } else {
                courseIntroduce.setSecondTop(0);
            }
            courseIntroduceRepository.save(courseIntroduce);
        }
    }

    public void saveOrUpdate(CourseIntroduceOperateDto courseIntroduceOperateDto) {
        CourseIntroduce _CourseIntroduce = new CourseIntroduce();
        _CourseIntroduce.setGmtModified(new Date());
        if (StringUtils.isEmpty(String.valueOf(courseIntroduceOperateDto.getId())) | courseIntroduceOperateDto.getId() == null) {
            _CourseIntroduce.setGmtCreated(new Date());
            _CourseIntroduce.setIsDel(0);
            _CourseIntroduce.setSeqencing(0);
        } else {
            Optional<CourseIntroduce> optional = courseIntroduceRepository.findById(courseIntroduceOperateDto.getId());
            if (!optional.isPresent()) {
                throw new RuntimeException("数据异常...");
            }
            _CourseIntroduce = optional.get();
        }

        _CourseIntroduce.setImageUrl(courseIntroduceOperateDto.getImageUrl());
        _CourseIntroduce.setCourseTitle(courseIntroduceOperateDto.getCourseTitle());
        _CourseIntroduce.setCourseUrl(courseIntroduceOperateDto.getCourseUrl());
        _CourseIntroduce.setContent(courseIntroduceOperateDto.getContent());
        _CourseIntroduce.setIntro(courseIntroduceOperateDto.getIntro());
        _CourseIntroduce.setSecondTop(courseIntroduceOperateDto.getSecondTop());
        _CourseIntroduce.setPushIndex(courseIntroduceOperateDto.getPushIndex());
        _CourseIntroduce.setReleaseStatue(courseIntroduceOperateDto.getReleaseStatue());
        if (courseIntroduceOperateDto.getReleaseStatue() == null) {
            _CourseIntroduce.setReleaseStatue(0);
        }
        if (courseIntroduceOperateDto.getSecondTop() == null) {
            _CourseIntroduce.setSecondTop(0);
        }
        if (courseIntroduceOperateDto.getPushIndex() == null) {
            _CourseIntroduce.setPushIndex(0);
        }
        courseIntroduceRepository.save(_CourseIntroduce);
    }

    public void delete(ListDto list) {
        List<String> idList = list.getIdList();
        for (String i : idList) {
            courseIntroduceRepository.fakedel(Long.valueOf(i));
        }
    }

    @Transactional
    public void changeSeqencing(long id, Integer seqencing) {
        Optional<CourseIntroduce> byId = courseIntroduceRepository.findById(id);
        CourseIntroduce courseIntroduce = byId.get();
        List<CourseIntroduce> _courseBySeqencing = courseIntroduceRepository.findBySeqencing(seqencing);
        if(_courseBySeqencing.size() != 0){
            CourseIntroduce courseBySeqencing = _courseBySeqencing.get(0);
            courseBySeqencing.setSeqencing(courseIntroduce.getSeqencing());
            courseIntroduceRepository.save(courseBySeqencing);
        }
        courseIntroduce.setSeqencing(seqencing);
        courseIntroduceRepository.save(courseIntroduce);
    }

}
