package cn.com.sailfish.linghang.service.admin;

import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;

import static org.springframework.util.FileCopyUtils.BUFFER_SIZE;

@Service
public class FileUploadAdminService {

    @Value("${spring.resources.static-locations}")
    private List<String> url;

    @Value("${sailfish.file.uploadAddress}")
    private String pppath;

    @Value("${sailfish.file.zipAddress}")
    private String zippath;

    @Value("${spring.mvc.static-path-pattern}")
    private List<String> pathPattern;

    String extFileName;
    String zipPath;
    String path;
    String prefix = "";

    public String savefile(MultipartFile file, String sourceType) {
        try {
            String oldFileName = file.getOriginalFilename();
            /* 计算文件重命名 */
             extFileName = oldFileName.substring(oldFileName.lastIndexOf(".")); // 截取后缀
            if (extFileName.equals(".png") | extFileName.equals(".jpeg") | extFileName.equals(".jpg")) {
                prefix = "/two-bar-codes";
            }
             path = pppath;
             zipPath = zippath;
            String dir = prefix + "/" + sourceType + "/" + new SimpleDateFormat("yyyy").
                    format(new Date()) + "/" + new SimpleDateFormat("MM").
                    format(new Date()) + "/";
            path = path + dir;
            /*zipPath = zippath + dir;*/
            if(prefix.equals("/two-bar-codes")) {
                return qr_savefile(file);
            }
            else{
                return zip_savefile(file);
            }

        } catch (Exception e) {
        }
        return "";
    }

    public String qr_savefile(MultipartFile file) throws  Exception{
        File _dir = new File(path); // 创建磁盘目录,等待输出文件
        if (!_dir.exists()) {
            _dir.mkdirs(); //tomcat下的webapp/ROOT/images/***/
        }
        String fileName = UUID.randomUUID().toString().replace("-", "") + extFileName;
        path = path + fileName;
        String databasepath = path;
        databasepath = "/vr_resource" + databasepath.substring(databasepath.lastIndexOf(prefix));
        file.transferTo(new File(path));
        return databasepath;
    }


    public String zip_savefile(MultipartFile file) throws  Exception{
        final File excelFile = File.createTempFile(String.valueOf(new Random().nextInt()), extFileName);
        file.transferTo(excelFile);
        String entityName = unZip(excelFile, zipPath);
        deleteFile(excelFile);
        return "/" + entityName + "html/index.html";
    }

    /**
     * zip解压
     *
     * @param srcFile     zip源文件
     * @param destDirPath 解压后的目标文件夹
     * @throws RuntimeException 解压失败会抛出运行时异常
     */

    public static String unZip(File srcFile, String destDirPath) throws RuntimeException {
        long start = System.currentTimeMillis();
        // 判断源文件是否存在
        if (!srcFile.exists()) {
            throw new RuntimeException(srcFile.getPath() + "所指文件不存在");
        }
        // 开始解压
        ZipFile zipFile = null;
        String entityName = "";
        int mark = 0;
        try {
            zipFile = new ZipFile(srcFile);
            Enumeration<?> entries = zipFile.entries();
            while (entries.hasMoreElements()) {
                ZipEntry entry = (ZipEntry) entries.nextElement();
                if(mark == 0){
                    entityName = entry.getName();
                    mark = 1;
                }
                System.out.println("解压" + entry.getName());
                // 如果是文件夹，就创建个文件夹
                if (entry.isDirectory()) {
                    String dirPath = destDirPath + "/" + entry.getName();
                    File dir = new File(dirPath);
                    dir.mkdirs();
                } else {
                    // 如果是文件，就先创建一个文件，然后用io流把内容copy过去
                    File targetFile = new File(destDirPath + "/" + entry.getName());
                    // 保证这个文件的父文件夹必须要存在
                    if (!targetFile.getParentFile().exists()) {
                        targetFile.getParentFile().mkdirs();
                    }
                    targetFile.createNewFile();
                    // 将压缩文件内容写入到这个文件中
                    InputStream is = zipFile.getInputStream(entry);
                    FileOutputStream fos = new FileOutputStream(targetFile);
                    int len;
                    byte[] buf = new byte[1024];
                    while ((len = is.read(buf)) != -1) {
                        fos.write(buf, 0, len);
                    }
                    // 关流顺序，先打开的后关闭
                    fos.close();
                    is.close();
                }
            }
            long end = System.currentTimeMillis();
            System.out.println("解压完成，耗时：" + (end - start) + " ms");
        } catch (Exception e) {
            throw new RuntimeException("unzip error from ZipUtils", e);
        } finally {

            if (zipFile != null) {
                try {
                    zipFile.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
            return entityName;
        }
    }

    /**
     * 删除
     *
     * @param files
     */
    private void deleteFile(File... files) {
        for (File file : files) {
            if (file.exists()) {
                file.delete();
            }
        }
    }
}
