package cn.com.sailfish.linghang.service.admin;

import cn.com.sailfish.linghang.dao.web.response.PageResponse;
import cn.com.sailfish.linghang.domain.ImageDetails;
import cn.com.sailfish.linghang.domain.OperateDto.ListDto;
import cn.com.sailfish.linghang.repository.ImageDetailsRepository;
import io.micrometer.core.instrument.util.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.domain.*;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.UUID;

@Service
public class ImageDetailsAdminService {

    @Value("${spring.resources.static-locations}")
    private List<String> url;

   @Value("${sailfish.file.uploadAddress}")
    private String pppath;

    @Value("${spring.mvc.static-path-pattern}")
    private List<String> pathPattern;

    @Autowired
    private ImageDetailsRepository imageDetailsRepository;

    public String saveImage(MultipartFile file, String sourceType, String type) {
        try {
            long size = file.getSize();
            if(size > 5242880) { throw new RuntimeException("上传图片大小超过5MB") ;}

            String path = pppath;
            String dir = "/" + "images" + "/" + sourceType + "/" + new SimpleDateFormat("yyyy").
                    format(new Date()) + "/" + new SimpleDateFormat("MM").
                    format(new Date()) + "/";
            path = path + dir; // 拼接磁盘路径,拼接url路径
            File _dir = new File(path); // 创建磁盘目录,等待输出文件
            if (!_dir.exists()) {
                _dir.mkdirs(); //tomcat下的webapp/ROOT/images/***/
            }


            /*        path=path.substring(path.lastIndexOf("/file/images/"));*/
            /*
             * 根据账号获取id
             * 存入图片路径地址到数据库
             */
            // 判断后缀,获取原名称,1.jpg
            String oldFileName = file.getOriginalFilename();

            ImageDetails _imageDetails = new ImageDetails();
            _imageDetails.setName(oldFileName);
            _imageDetails.setSize(file.getSize());
            _imageDetails.setIsDel(0);
            _imageDetails.setSourceType(sourceType);
            _imageDetails.setType(Integer.valueOf(type));

            /* 计算文件重命名 */
            String extFileName = oldFileName.substring(oldFileName.lastIndexOf(".")); // 截取后缀
            String fileName = UUID.randomUUID().toString().replace("-", "") + extFileName;

            path = path + fileName;
            String databasepath = path;
            databasepath = databasepath.substring(databasepath.lastIndexOf("/images"));
            _imageDetails.setPath(databasepath);
            imageDetailsRepository.save(_imageDetails);

            //生成图片在本地
            file.transferTo(new File(path));
            return databasepath;
        } catch (Exception e) {
        }
        return "";
    }

    public void deleteImage(ListDto list) {
        List<String> idList = list.getIdList();
        for (String i : idList) {
            imageDetailsRepository.fakedel(Long.valueOf(i));
        }
    }

    public PageResponse<ImageDetails> findAllByStatue(Integer page, Integer pageSize, String crux, List<String> sourceType) {
        ImageDetails _imageDetails = new ImageDetails();
        _imageDetails.setIsDel(0);

        Sort sort = new Sort(Sort.Direction.DESC, "gmtCreated");
        if (!StringUtils.isEmpty(crux)) {
            _imageDetails.setName(crux);
            Example<ImageDetails> example = Example.of(_imageDetails, ExampleMatcher.matching().withMatcher("name", ExampleMatcher.GenericPropertyMatchers.contains()));
            Pageable pageable = PageRequest.of(page - 1, pageSize, sort);
            if (!CollectionUtils.isEmpty(sourceType)) {
                Page<ImageDetails> imageDetails = imageDetailsRepository.findAllbysourceTypeAndCrux(crux, pageable, sourceType);
                return courseIntroducePageResponse(imageDetails);
            } else {
                Page<ImageDetails> imageDetails = imageDetailsRepository.findAll(example, pageable);
                return courseIntroducePageResponse(imageDetails);
            }
        }

        if (!CollectionUtils.isEmpty(sourceType)) {
            Pageable pageable = PageRequest.of(page - 1, pageSize, sort);
            Page<ImageDetails> imageDetails = imageDetailsRepository.findAllbysourceType(pageable, sourceType);
            return courseIntroducePageResponse(imageDetails);
        }

        Example<ImageDetails> example = Example.of(_imageDetails);
        Pageable pageable = PageRequest.of(page - 1, pageSize, sort);
        Page<ImageDetails> imageDetails = imageDetailsRepository.findAll(example, pageable);
        return courseIntroducePageResponse(imageDetails);
    }

    public synchronized PageResponse<ImageDetails> courseIntroducePageResponse(Page<ImageDetails> newscourseIntroduce) {
        PageResponse pageResponse = new PageResponse();
        if (newscourseIntroduce != null) {
            pageResponse.setPageIndex(newscourseIntroduce.getPageable().getPageNumber() + 1);
            pageResponse.setPageSize(newscourseIntroduce.getPageable().getPageSize());
            pageResponse.setTotalCount(newscourseIntroduce.getTotalElements());
            pageResponse.setTotalPageCount(newscourseIntroduce.getTotalPages());
            pageResponse.setList(newscourseIntroduce.getContent());
        }
        return pageResponse;
    }

}
