package cn.com.sailfish.linghang.service.admin;

import cn.com.sailfish.linghang.dao.web.response.PageResponse;
import cn.com.sailfish.linghang.domain.NewsPaper;
import cn.com.sailfish.linghang.domain.OperateDto.ListDto;
import cn.com.sailfish.linghang.domain.OperateDto.NewsPaperOperateDto;
import cn.com.sailfish.linghang.repository.NewsPaperRepository;
import io.micrometer.core.instrument.util.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.*;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Optional;

@Service
public class NewsPaperAdminService {

    @Autowired
    private NewsPaperRepository newsPaperRepository;

    /**
     * @param id
     * @return
     * @description 根据ID查询
     */
    public NewsPaper findById(Long id) {
        return newsPaperRepository.findnewsPaperById(id);
    }

    /**
     * @param page
     * @param pageSize
     * @return
     * @description 分页查询
     */
    public PageResponse<NewsPaper> findAllByStatueAndId(Integer page, Integer pageSize, String crux) {
        NewsPaper newsPaper = new NewsPaper();
        newsPaper.setIsDel(0);
        List<Sort.Order> orders = new ArrayList<>();
        orders.add(new Sort.Order(Sort.Direction.DESC, "releaseStatue"));
        orders.add(new Sort.Order(Sort.Direction.DESC, "pushIndex"));
        orders.add(new Sort.Order(Sort.Direction.DESC, "secondTop"));
        orders.add(new Sort.Order(Sort.Direction.ASC, "seqencing"));
        orders.add(new Sort.Order(Sort.Direction.DESC, "gmtCreated"));

        if (!StringUtils.isEmpty(crux)) {
            newsPaper.setNewsTitle(crux);
            Example<NewsPaper> example = Example.of(newsPaper, ExampleMatcher.matching().withMatcher("newsTitle", ExampleMatcher.GenericPropertyMatchers.contains()));
            Pageable pageable = PageRequest.of(page - 1, pageSize, new Sort(orders));
            Page<NewsPaper> NewsPaper = newsPaperRepository.findAll(example, pageable);
            return newsPaperResponsePageResponse(NewsPaper);
        }

        Example<NewsPaper> example = Example.of(newsPaper);
        Pageable pageable = PageRequest.of(page - 1, pageSize, new Sort(orders));
        Page<NewsPaper> NewsPaperPage = newsPaperRepository.findAll(example, pageable);

        return newsPaperResponsePageResponse(NewsPaperPage);
    }

    /**
     * @param id
     * @description 前移方法
     */
    public void forward(Long id) {
        NewsPaper newspaper = newsPaperRepository.findByIdAndSeqencing(id);
        if (newspaper != null) {
            newsPaperRepository.forward(id);
        }
    }

    /**
     * description 后移方法
     *
     * @param id
     * @return
     */
    public void backward(Long id) {
        newsPaperRepository.backward(id);
    }

    /**
     * @param id
     * @return
     * @description 新闻发布；
     */
    public void checkboxrelease(Long id) {
        NewsPaper newspaper = newsPaperRepository.findById(id).orElse(null);
        if (newspaper != null) {
            newspaper.setReleaseStatue(1);
        }
        newsPaperRepository.save(newspaper);
    }

    /**
     * @param id
     * @return
     * @description 取消发布
     */
    public void checkboxnorelease(Long id) {
        NewsPaper newspaper = newsPaperRepository.findById(id).orElse(null);
        if (newspaper != null) {
            newspaper.setReleaseStatue(0);
        }
        newsPaperRepository.save(newspaper);
    }

    public synchronized PageResponse<NewsPaper> newsPaperResponsePageResponse(Page<NewsPaper> newsPaperPage) {
        PageResponse<NewsPaper> pageResponse = new PageResponse<>();
        if (newsPaperPage != null) {
            pageResponse.setPageIndex(newsPaperPage.getPageable().getPageNumber() + 1);
            pageResponse.setPageSize(newsPaperPage.getPageable().getPageSize());
            pageResponse.setTotalCount(newsPaperPage.getTotalElements());
            pageResponse.setTotalPageCount(newsPaperPage.getTotalPages());
            pageResponse.setList(newsPaperPage.getContent());
        }

        return pageResponse;
    }

    public void saveOrUpdate(NewsPaperOperateDto newsPaperOperateDto) {
        NewsPaper _newsPaper = new NewsPaper();
        _newsPaper.setGmtModified(new Date());
        if (StringUtils.isEmpty(String.valueOf(newsPaperOperateDto.getId())) | newsPaperOperateDto.getId() == null) {
            _newsPaper.setGmtCreated(new Date());
            _newsPaper.setIsDel(0);
            _newsPaper.setSeqencing(0);
        } else {
            Optional<NewsPaper> optional = newsPaperRepository.findById(newsPaperOperateDto.getId());
            if (!optional.isPresent()) {
                throw new RuntimeException("数据异常...");
            }

            _newsPaper = optional.get();
        }
        _newsPaper.setImageUrl(newsPaperOperateDto.getImageUrl());
        _newsPaper.setNewsTitle(newsPaperOperateDto.getNewsTitle());
        _newsPaper.setNewUrl(newsPaperOperateDto.getNewUrl());
        _newsPaper.setContent(newsPaperOperateDto.getContent());
        _newsPaper.setIntro(newsPaperOperateDto.getIntro());
        _newsPaper.setClasstype(newsPaperOperateDto.getClasstype());
        _newsPaper.setReleaseStatue(newsPaperOperateDto.getReleaseStatue());
        _newsPaper.setSecondTop(newsPaperOperateDto.getSecondTop());
        _newsPaper.setPushIndex(newsPaperOperateDto.getPushIndex());
        if (newsPaperOperateDto.getClasstype() == null) {
            _newsPaper.setClasstype(1);
        }
        if (newsPaperOperateDto.getReleaseStatue() == null) {
            _newsPaper.setReleaseStatue(0);
        }
        if (newsPaperOperateDto.getSecondTop() == null) {
            _newsPaper.setSecondTop(0);
        }
        if (newsPaperOperateDto.getPushIndex() == null) {
            _newsPaper.setPushIndex(0);
        }
        newsPaperRepository.save(_newsPaper);
    }

    public void delete(ListDto list) {
        List<String> idList = list.getIdList();

        for (String i : idList) {
            newsPaperRepository.fakedel(Long.valueOf(i));
        }
    }

    public void changePushIndex(Long id) {
        List<NewsPaper> newsPaperByPushIndex = newsPaperRepository.findNewsPaperByPushIndex();

        NewsPaper newsPaper = newsPaperRepository.findnewsPaperById(id);
        if (newsPaper != null) {
            if (newsPaper.getPushIndex() == 0) {
                if (newsPaperByPushIndex.size() >= 3) {
                    throw new RuntimeException("强推不能拥有大于3条记录");
                }
                newsPaper.setPushIndex(1);
            } else {
                newsPaper.setPushIndex(0);
            }
            newsPaperRepository.save(newsPaper);
        }
    }

    public void changeSecondTop(Long id) {
        NewsPaper newsPaper = newsPaperRepository.findnewsPaperById(id);
        if (null != newsPaper) {
            newsPaper.setSecondTop(newsPaper.getSecondTop() == 0 ? 1 : 0);
            newsPaperRepository.save(newsPaper);
        }
    }

    public void changeClasstype(Long id) {
        NewsPaper newsPaper = newsPaperRepository.findnewsPaperById(id);
        if (null != newsPaper) {
            newsPaper.setClasstype(newsPaper.getClasstype() == 1 ? 2 : 1);
            newsPaperRepository.save(newsPaper);
        }
    }

    @Transactional
    public void changeSeqencing(long id, Integer seqencing) {
        Optional<NewsPaper> byId = newsPaperRepository.findById(id);
        NewsPaper newsPaper = byId.get();
        List<NewsPaper> _newsPaperBySeqencing = newsPaperRepository.findBySeqencing(seqencing);
        if (_newsPaperBySeqencing.size() != 0) {
            NewsPaper newsPaperBySeqencing = _newsPaperBySeqencing.get(0);
            newsPaperBySeqencing.setSeqencing(newsPaper.getSeqencing());
            newsPaperRepository.save(newsPaperBySeqencing);
        }
        newsPaper.setSeqencing(seqencing);
        newsPaperRepository.save(newsPaper);
    }
}
