package cn.com.sailfish.linghang.service.admin;

import cn.com.sailfish.linghang.dao.web.response.PageResponse;
import cn.com.sailfish.linghang.domain.OperateDto.ListDto;
import cn.com.sailfish.linghang.domain.OperateDto.ThdModelOperateDto;
import cn.com.sailfish.linghang.domain.ThdModel;
import cn.com.sailfish.linghang.repository.ThdModelRepository;
import io.micrometer.core.instrument.util.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.*;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Optional;

@Service
public class TheModelAdminService {

    @Autowired
    private ThdModelRepository thdModelRepository;

    private static final Logger log = LoggerFactory.getLogger(TheModelAdminService.class);

    /**
     * @param page
     * @param pageSize
     * @return
     * @description 分页
     */
    public PageResponse<ThdModel> pagebystatue(Integer page, Integer pageSize, String crux) {

        ThdModel thdModel = new ThdModel();

        thdModel.setIsDel(0);

        ExampleMatcher matcher;

        List<Sort.Order> orders = new ArrayList<>();
        orders.add(new Sort.Order(Sort.Direction.DESC, "releaseStatue"));
        orders.add(new Sort.Order(Sort.Direction.DESC, "pushIndex"));
        orders.add(new Sort.Order(Sort.Direction.DESC, "secondTop"));
        orders.add(new Sort.Order(Sort.Direction.ASC, "seqencing"));
        orders.add(new Sort.Order(Sort.Direction.DESC, "gmtCreated"));

        if (!StringUtils.isEmpty(crux)) {
            thdModel.setThdTitle(crux);
            Example<ThdModel> example = Example.of(thdModel, ExampleMatcher.matching().withMatcher("thdTitle", ExampleMatcher.GenericPropertyMatchers.contains()));
            Pageable pageable = PageRequest.of(page - 1, pageSize, new Sort(orders));
            Page<ThdModel> themodel = thdModelRepository.findAll(example, pageable);
            return thdModelpageResponse(themodel);

        }
        Example<ThdModel> example = Example.of(thdModel);
        Pageable pageable = PageRequest.of(page - 1, pageSize, new Sort(orders));
        Page<ThdModel> themodel = thdModelRepository.findAll(example, pageable);
        return thdModelpageResponse(themodel);
    }

    /**
     * @param id
     * @description 前移方法
     */
    public void forward(long id) {
        ThdModel newspaper = thdModelRepository.findByIdAndSeqencing(id);
        if (newspaper != null) {
            thdModelRepository.forward(id);
        }
    }

    /**
     * description 后移方法
     *
     * @param id
     * @return
     */
    public void backward(long id) {
        thdModelRepository.backward(id);
    }

    /**
     * @param id
     * @return
     * @description 新闻发布；
     */
    public void checkboxrelease(long id) {
        ThdModel thdModel = thdModelRepository.findById(id).orElse(null);
        if (thdModel != null) {
            thdModel.setReleaseStatue(1);
        }
        thdModelRepository.save(thdModel);
    }

    /**
     * @param id
     * @return
     * @description 取消发布
     */
    public void checkboxnorelease(long id) {
        ThdModel thdModel = thdModelRepository.findById(id).orElse(null);
        if (thdModel != null) {
            thdModel.setReleaseStatue(0);
        }
        thdModelRepository.save(thdModel);
    }

    public synchronized PageResponse<ThdModel> thdModelpageResponse(Page<ThdModel> thdModelPaperPage) {

        PageResponse<ThdModel> pageResponse = new PageResponse<>();
        if (thdModelPaperPage != null) {
            pageResponse.setPageIndex(thdModelPaperPage.getPageable().getPageNumber() + 1);
            pageResponse.setPageSize(thdModelPaperPage.getPageable().getPageSize());
            pageResponse.setTotalCount(thdModelPaperPage.getTotalElements());
            pageResponse.setTotalPageCount(thdModelPaperPage.getTotalPages());
            pageResponse.setList(thdModelPaperPage.getContent());
        }

        return pageResponse;
    }

    public ThdModel findById(String id) {
        return thdModelRepository.findThdModelById(Long.valueOf(id));
    }

    public void changePushIndex(String id) {
        List<ThdModel> modelByPushIndex = thdModelRepository.findModelByPushIndex();

        ThdModel tmodel = thdModelRepository.findThdModelById(Long.valueOf(id));
        if (tmodel != null) {
            if (tmodel.getPushIndex() == 0) {
                if(modelByPushIndex.size() >=4 ) {
                    throw new RuntimeException("强推不能拥有大于4条记录");
                }
                tmodel.setPushIndex(1);
            } else {
                tmodel.setPushIndex(0);
            }
            thdModelRepository.save(tmodel);
        }
    }

    public void changeSecondTop(String id) {
        ThdModel tmodel = thdModelRepository.findThdModelById(Long.valueOf(id));
        if (tmodel != null) {
            if (tmodel.getSecondTop() == 0) {
                tmodel.setSecondTop(1);
            } else {
                tmodel.setSecondTop(0);
            }
            thdModelRepository.save(tmodel);
        }
    }

    public void saveOrUpdate(ThdModelOperateDto thdModelOperateDto) {
        ThdModel _thdModel = new ThdModel();
        _thdModel.setGmtModified(new Date());
        if (StringUtils.isEmpty(String.valueOf(thdModelOperateDto.getId())) | thdModelOperateDto.getId() == null) {
            _thdModel.setGmtCreated(new Date());
            _thdModel.setIsDel(0);
            _thdModel.setSeqencing(0);
        } else {
            Optional<ThdModel> optional = thdModelRepository.findById(thdModelOperateDto.getId());
            if (!optional.isPresent()) {
                throw new RuntimeException("数据异常...");
            }
            _thdModel = optional.get();
        }
        _thdModel.setImageUrl(thdModelOperateDto.getImageUrl());
        _thdModel.setThdTitle(thdModelOperateDto.getThdTitle());
        _thdModel.setImageUrl(thdModelOperateDto.getImageUrl());
        _thdModel.setIntro(thdModelOperateDto.getIntro());
        _thdModel.setSecondTop(thdModelOperateDto.getSecondTop());
        _thdModel.setPushIndex(thdModelOperateDto.getPushIndex());
        _thdModel.setReleaseStatue(thdModelOperateDto.getReleaseStatue());
        _thdModel.setQrcodeUrl(thdModelOperateDto.getQrcodeUrl());
        _thdModel.setZipUrl(thdModelOperateDto.getZipUrl());
        if (thdModelOperateDto.getReleaseStatue() == null) {
            _thdModel.setReleaseStatue(0);
        }
        if (thdModelOperateDto.getSecondTop() == null) {
            _thdModel.setSecondTop(0);
        }
        if (thdModelOperateDto.getPushIndex() == null) {
            _thdModel.setPushIndex(0);
        }
        thdModelRepository.save(_thdModel);
    }

    public void delete(ListDto list) {
        List<String> idList = list.getIdList();
        for (String i :
                idList) {
            thdModelRepository.fakedel(Long.valueOf(i));
        }


    }


    @Transactional
    public void changeSeqencing(long id, Integer seqencing) {
        Optional<ThdModel> byId = thdModelRepository.findById(id);
        ThdModel thdModel = byId.get();
        List<ThdModel> _modelBySeqencing = thdModelRepository.findBySeqencing(seqencing);
        if(_modelBySeqencing.size() != 0){
            ThdModel modelBySeqencing = _modelBySeqencing.get(0);
            modelBySeqencing.setSeqencing(thdModel.getSeqencing());
            thdModelRepository.save(modelBySeqencing);
        }
        thdModel.setSeqencing(seqencing);
        thdModelRepository.save(thdModel);
    }
}
