package cn.com.sailfish.linghang.service.admin;

import cn.com.sailfish.linghang.dao.web.response.PageResponse;
import cn.com.sailfish.linghang.domain.OperateDto.ListDto;
import cn.com.sailfish.linghang.domain.OperateDto.VrDetailsOperateDto;
import cn.com.sailfish.linghang.domain.VrDetails;
import cn.com.sailfish.linghang.repository.VrPanoramaRepository;
import io.micrometer.core.instrument.util.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.*;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Optional;

@Service
public class VrPanoramAdminService {
    @Autowired
    private VrPanoramaRepository vrPanoramaRepository;


    public PageResponse<VrDetails> pagebystatue(Integer page, Integer pageSize, String crux) {

        VrDetails vrDetails = new VrDetails();
        vrDetails.setIsDel(0);

        ExampleMatcher matcher;

        List<Sort.Order> orders = new ArrayList<>();
        orders.add(new Sort.Order(Sort.Direction.DESC, "releaseStatue"));
        orders.add(new Sort.Order(Sort.Direction.DESC, "pushIndex"));
        orders.add(new Sort.Order(Sort.Direction.DESC, "secondTop"));
        orders.add(new Sort.Order(Sort.Direction.ASC, "seqencing"));
        orders.add(new Sort.Order(Sort.Direction.DESC, "gmtCreated"));

        if (!StringUtils.isEmpty(crux)) {
            vrDetails.setVrTitle(crux);
            Example<VrDetails> example = Example.of(vrDetails, ExampleMatcher.matching().withMatcher("vrTitle", ExampleMatcher.GenericPropertyMatchers.contains()));
            Pageable pageable = PageRequest.of(page - 1, pageSize, new Sort(orders));
            Page<VrDetails> vrDetail = vrPanoramaRepository.findAll(example, pageable);
            return vrDetailsPageResponse(vrDetail);

        }
        Example<VrDetails> example = Example.of(vrDetails);
        Pageable pageable = PageRequest.of(page - 1, pageSize, new Sort(orders));
        Page<VrDetails> vrDetailPage = vrPanoramaRepository.findAll(example, pageable);
        return vrDetailsPageResponse(vrDetailPage);
    }

    /**
     * @param id
     * @description 前移方法
     */
    public void forward(long id) {
        VrDetails newspaper = vrPanoramaRepository.findByIdAndSeqencing(id);
        if (newspaper != null) {
            vrPanoramaRepository.forward(id);
        }
    }

    /**
     * description 后移方法
     *
     * @param id
     * @return
     */
    public void backward(long id) {
        vrPanoramaRepository.backward(id);
    }

    /**
     * @param id
     * @return
     * @description 新闻发布；
     */
    public void checkboxrelease(long id) {
        VrDetails vrDetails = vrPanoramaRepository.findById(id).orElse(null);
        if (vrDetails != null) {
            vrDetails.setReleaseStatue(1);
        }
        vrPanoramaRepository.save(vrDetails);
    }

    /**
     * @param id
     * @return
     * @description 取消发布
     */
    public void checkboxnorelease(long id) {
        VrDetails vrDetails = vrPanoramaRepository.findById(id).orElse(null);
        if (vrDetails != null) {
            vrDetails.setReleaseStatue(0);
        }
        vrPanoramaRepository.save(vrDetails);
    }

    public synchronized PageResponse<VrDetails> vrDetailsPageResponse(Page<VrDetails> VrDetailsPaperPage) {

        PageResponse<VrDetails> pageResponse = new PageResponse<>();
        if (VrDetailsPaperPage != null) {
            pageResponse.setPageIndex(VrDetailsPaperPage.getPageable().getPageNumber() + 1);
            pageResponse.setPageSize(VrDetailsPaperPage.getPageable().getPageSize());
            pageResponse.setTotalCount(VrDetailsPaperPage.getTotalElements());
            pageResponse.setTotalPageCount(VrDetailsPaperPage.getTotalPages());
            pageResponse.setList(VrDetailsPaperPage.getContent());
        }

        return pageResponse;
    }

    public VrDetails findById(String id) {
        return vrPanoramaRepository.findVrDetailsById(Long.valueOf(id));
    }

    public void changePushIndex(String id) {
        List<VrDetails> vrDetailsByPushIndex = vrPanoramaRepository.findVrDetailsByPushIndex();

        VrDetails vrDetail = vrPanoramaRepository.findVrDetailsById(Long.valueOf(id));
        if (vrDetail != null) {
            if (vrDetail.getPushIndex() == 0) {
                if(vrDetailsByPushIndex.size() >=4 ) {
                    throw new RuntimeException("强推不能拥有大于4条记录");
                }
                vrDetail.setPushIndex(1);
            } else {
                vrDetail.setPushIndex(0);
            }
            vrPanoramaRepository.save(vrDetail);
        }
    }

    public void changeSecondTop(String id) {
        VrDetails vrDetail = vrPanoramaRepository.findVrDetailsById(Long.valueOf(id));
        if (vrDetail != null) {
            if (vrDetail.getSecondTop() == 0) {
                vrDetail.setSecondTop(1);
            } else {
                vrDetail.setSecondTop(0);
            }
            vrPanoramaRepository.save(vrDetail);
        }
    }

    public void saveOrUpdate(VrDetailsOperateDto vrDetailsOperateDto) {
        VrDetails _vrDetails = new VrDetails();
        _vrDetails.setGmtModified(new Date());
        if (StringUtils.isEmpty(String.valueOf(vrDetailsOperateDto.getId())) | vrDetailsOperateDto.getId() == null) {
            _vrDetails.setGmtCreated(new Date());
            _vrDetails.setIsDel(0);
            _vrDetails.setSeqencing(0);
        } else {
            Optional<VrDetails> optional = vrPanoramaRepository.findById(vrDetailsOperateDto.getId());
            if (!optional.isPresent()) {
                throw new RuntimeException("数据异常...");
            }
            _vrDetails = optional.get();
        }
        _vrDetails.setImageUrl(vrDetailsOperateDto.getImageUrl());
        _vrDetails.setVrTitle(vrDetailsOperateDto.getVrTitle());
        _vrDetails.setImageUrl(vrDetailsOperateDto.getImageUrl());
        _vrDetails.setIntro(vrDetailsOperateDto.getIntro());
        _vrDetails.setSecondTop(vrDetailsOperateDto.getSecondTop());
        _vrDetails.setPushIndex(vrDetailsOperateDto.getPushIndex());
        _vrDetails.setReleaseStatue(vrDetailsOperateDto.getReleaseStatue());
        _vrDetails.setQrcodeUrl(vrDetailsOperateDto.getQrcodeUrl());
        _vrDetails.setZipUrl(vrDetailsOperateDto.getZipUrl());
        if (vrDetailsOperateDto.getReleaseStatue() == null) {
            _vrDetails.setReleaseStatue(0);
        }
        if (vrDetailsOperateDto.getSecondTop() == null) {
            _vrDetails.setSecondTop(0);
        }
        if (vrDetailsOperateDto.getPushIndex() == null) {
            _vrDetails.setPushIndex(0);
        }
        vrPanoramaRepository.save(_vrDetails);
    }

    public void delete(ListDto list) {
        List<String> idList = list.getIdList();
        for (String i :
                idList) {
            vrPanoramaRepository.fakedel(Long.valueOf(i));
        }


    }

    @Transactional
    public void changeSeqencing(long id, Integer seqencing) {
        Optional<VrDetails> byId = vrPanoramaRepository.findById(id);
        VrDetails vrDetails = byId.get();
        List<VrDetails> _vrDetailsBySeqencing = vrPanoramaRepository.findBySeqencing(seqencing);
        if (_vrDetailsBySeqencing.size() != 0) {
            VrDetails vrDetailsPaperBySeqencing = _vrDetailsBySeqencing.get(0);
            vrDetailsPaperBySeqencing.setSeqencing(vrDetails.getSeqencing());
            vrPanoramaRepository.save(vrDetailsPaperBySeqencing);
        }
        vrDetails.setSeqencing(seqencing);
        vrPanoramaRepository.save(vrDetails);
    }
}
