package cn.com.sailfish.linghang.service.portal;

import cn.com.sailfish.linghang.dao.web.response.PageResponse;
import cn.com.sailfish.linghang.domain.NewsPaper;
import cn.com.sailfish.linghang.repository.NewsPaperRepository;
import io.micrometer.core.instrument.util.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.*;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

@Service
public class NewsPaperPortalService {

    @Autowired
    private NewsPaperRepository newsPaperRepository;

    private static final Logger log = LoggerFactory.getLogger(NewsPaperPortalService.class);


    /**
     * @return
     * @description 查询所有新闻
     */
    public List<NewsPaper> findAll() {
        NewsPaper newsPaper = new NewsPaper();
        newsPaper.setIsDel(0);
        newsPaper.setReleaseStatue(1);
        Example<NewsPaper> example = Example.of(newsPaper, ExampleMatcher.matching());
        Sort sort = new Sort(Sort.Direction.DESC, "pushIndex")
                .and(new Sort(Sort.Direction.ASC, "seqencing"))
                .and(new Sort(Sort.Direction.DESC, "gmtCreated"));
        return newsPaperRepository.findAll(example, sort);
    }


    /**
     * @param Id
     * @return
     * @description 根据ID查询
     */
    public NewsPaper findById(String Id) {
        return newsPaperRepository.findById(Long.valueOf(Id)).orElse(null);
    }

    /**
     * @param page
     * @param pageSize
     * @return
     * @description 分页查询
     */
    public PageResponse<NewsPaper> findAllByStatueAndId(Integer page, Integer pageSize, String crux, String newstype) {

        NewsPaper newsPaper = new NewsPaper();

        newsPaper.setIsDel(0);
        newsPaper.setReleaseStatue(1);
        ExampleMatcher matcher;

        List<Sort.Order> orders = new ArrayList<>();
        orders.add(new Sort.Order(Sort.Direction.DESC, "secondTop"));
        orders.add(new Sort.Order(Sort.Direction.ASC, "seqencing"));
        orders.add(new Sort.Order(Sort.Direction.DESC, "gmtCreated"));

        if (!StringUtils.isEmpty(crux)) {

            Pageable pageable = PageRequest.of(page - 1, pageSize, new Sort(orders));
            if (newstype != null) {
                switch (newstype) {
                    case "1":
                        Page<NewsPaper> newsPaperPage = newsPaperRepository.findpage(crux, pageable);
                        return newsPaperResponsePageResponse(newsPaperPage);
                    case "2":
                        Page<NewsPaper> findpagexinwen = newsPaperRepository.findpagexinwen(crux, pageable);
                        return newsPaperResponsePageResponse(findpagexinwen);
                    case "3":
                        Page<NewsPaper> findpagehangye = newsPaperRepository.findpagehangye(crux, pageable);
                        return newsPaperResponsePageResponse(findpagehangye);

                }
            } else {
                Page<NewsPaper> newsPaperPage = newsPaperRepository.findpage(crux, pageable);
                return newsPaperResponsePageResponse(newsPaperPage);
            }


        }
        Example<NewsPaper> example = Example.of(newsPaper);
        Pageable pageable = PageRequest.of(page - 1, pageSize, new Sort(orders));
        Page<NewsPaper> NewsPaperPage = newsPaperRepository.findAll(example, pageable);
        return newsPaperResponsePageResponse(NewsPaperPage);

    }

    public synchronized PageResponse<NewsPaper> newsPaperResponsePageResponse(Page<NewsPaper> newsPaperPage) {

        PageResponse<NewsPaper> pageResponse = new PageResponse<>();
        if (newsPaperPage != null) {
            pageResponse.setPageIndex(newsPaperPage.getPageable().getPageNumber() + 1);
            pageResponse.setPageSize(newsPaperPage.getPageable().getPageSize());
            pageResponse.setTotalCount(newsPaperPage.getTotalElements());
            pageResponse.setTotalPageCount(newsPaperPage.getTotalPages());
            pageResponse.setList(newsPaperPage.getContent());
        }

        return pageResponse;
    }

}
