package cn.com.sailfish.linghang.util;

import cn.com.sailfish.linghang.security.JWTUser;
import cn.com.sailfish.linghang.security.SystemPrivilege;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContext;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.core.userdetails.UserDetails;

/**
 * Author: huangpeilin
 * Create at: 2018-12-12 14:27:39
 * Description:
 *
 * @author huangpeilin
 */
public final class SecurityUtil {

    public static String getCurrentUserLogin() {
        SecurityContext securityContext = SecurityContextHolder.getContext();
        Authentication authentication = securityContext.getAuthentication();
        String loginName = null;
        if (authentication != null) {
            if (authentication.getPrincipal() instanceof UserDetails) {
                UserDetails springSecurityUser = (UserDetails) authentication.getPrincipal();
                loginName = springSecurityUser.getUsername();
            } else if (authentication.getPrincipal() instanceof String) {
                loginName = (String) authentication.getPrincipal();
            }
        }
        return loginName;
    }

    public static String getCurrentUserJWT() {
        SecurityContext securityContext = SecurityContextHolder.getContext();
        Authentication authentication = securityContext.getAuthentication();
        if (authentication != null && authentication.getCredentials() instanceof String) {
            return (String) authentication.getCredentials();
        }
        return null;
    }

    public static boolean isAuthenticated() {
        SecurityContext securityContext = SecurityContextHolder.getContext();
        Authentication authentication = securityContext.getAuthentication();
        if (authentication != null) {
            if (authentication.getPrincipal() instanceof UserDetails) {
                JWTUser springSecurityUser = (JWTUser) authentication.getPrincipal();
                return springSecurityUser.getAuthority() == 0;
            } else {
                return authentication.getAuthorities().stream()
                        .noneMatch(grantedAuthority -> grantedAuthority.getAuthority().equals(SystemPrivilege.ANONYMOUS));
            }
        }
        return false;
    }

    public static boolean isCurrentUserInRole(int authority) {
        SecurityContext securityContext = SecurityContextHolder.getContext();
        Authentication authentication = securityContext.getAuthentication();
        if (authentication != null) {
            if (authentication.getPrincipal() instanceof UserDetails) {
                JWTUser springSecurityUser = (JWTUser) authentication.getPrincipal();
                return (springSecurityUser.getAuthority() & authority) > 0;
            } else {
                return authentication.getAuthorities().stream()
                        .anyMatch(grantedAuthority ->
                                grantedAuthority.getAuthority().equals(SystemPrivilege.getPrivilegeName(authority)));
            }
        }
        return false;
    }
}
