package cn.com.sailfish.linghang.util;

import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.Instant;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;
import java.util.Date;
import java.util.Locale;

/**
 * Author: huangpeilin
 * Create at: 2019-04-28 19:58:38
 * Description:
 *
 * @author huangpeilin
 */
public final class TimeUtil {

    /**
     * 获得某天最大时间 2017-10-15 23:59:59
     * @param time
     * @return
     */
    public static Instant getEndOfDay(Instant time) {
        LocalDateTime localDateTime = LocalDateTime.ofInstant(time, ZoneId.systemDefault());;
        LocalDateTime endOfDay = localDateTime.with(LocalTime.MAX);
        return endOfDay.atZone(ZoneId.systemDefault()).toInstant();
    }

    /**
     * 获得某天最小时间 2017-10-15 00:00:00
     * @param time
     * @return
     */
    public static Instant getStartOfDay(Instant time) {
        LocalDateTime localDateTime = LocalDateTime.ofInstant(time, ZoneId.systemDefault());
        LocalDateTime startOfDay = localDateTime.with(LocalTime.MIN);
        return startOfDay.atZone(ZoneId.systemDefault()).toInstant();
    }

    /**
     * 格式化时间
     * @param time
     * @return
     */
    public static String formattingTime(Instant time, String pattern) {
        String format = null;
        if (time == null){
            format = "";
        }else{
            DateTimeFormatter dateTimeFormatter = DateTimeFormatter.ofPattern(pattern);
            LocalDateTime localDateTime = LocalDateTime.ofInstant(time, ZoneId.systemDefault());
            format = dateTimeFormatter.format(localDateTime);
        }
        return format;
    }

    /**
     * 将Instant的时间转化为为毫秒时间戳
     * @param instant
     * @return
     */
    public static Long instantToTimestamp(Instant instant){
        if (instant != null){
            return instant.toEpochMilli();
        }
        return null;
    }

    /**
     * 两个时间相减得到分钟数
     * @param subtrahendTime 减数
     * @param minuendTime 被减数
     * @return
     */
    public static String subtractTime(Instant subtrahendTime, Instant minuendTime) {
        if (subtrahendTime == null || minuendTime == null){
            return "";
        }
        long subtrahend = subtrahendTime.toEpochMilli();
        long minuend = minuendTime.toEpochMilli();
        long result;
        if (subtrahend > minuend){
            result = subtrahend - minuend;
        }else{
            result = minuend - subtrahend;
        }

        int minute = (int)(result / (60 * 1000));
        return String.valueOf(minute);
    }

    public static String dealDateFormat(String oldDate) {
        Date date1 = null;
        DateFormat df2 = null;
        try {
            DateFormat df = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss");
            Date date = df.parse(oldDate);
            SimpleDateFormat df1 = new SimpleDateFormat ("EEE MMM dd HH:mm:ss Z yyyy", Locale.UK);
            date1 = df1.parse(date.toString());
            df2 = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        } catch (ParseException e) {

            e.printStackTrace();
        }
        return df2.format(date1);
    }
}
